# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .address import Address
from .authorization_state import AuthorizationState
from .background import Background
from .basic_group import BasicGroup
from .basic_group_full_info import BasicGroupFullInfo
from .call import Call
from .callback_query_payload import CallbackQueryPayload
from .chat import Chat
from .chat_action import ChatAction
from .chat_action_bar import ChatActionBar
from .chat_filter_info import ChatFilterInfo
from .chat_invite_link import ChatInviteLink
from .chat_list import ChatList
from .chat_member import ChatMember
from .chat_nearby import ChatNearby
from .chat_notification_settings import ChatNotificationSettings
from .chat_permissions import ChatPermissions
from .chat_photo_info import ChatPhotoInfo
from .chat_position import ChatPosition
from .chat_type import ChatType
from .connection_state import ConnectionState
from .draft_message import DraftMessage
from .file import File
from .group_call import GroupCall
from .group_call_participant import GroupCallParticipant
from .language_pack_string import LanguagePackString
from .location import Location
from .message import Message
from .message_content import MessageContent
from .message_interaction_info import MessageInteractionInfo
from .notification import Notification
from .notification_group import NotificationGroup
from .notification_group_type import NotificationGroupType
from .notification_settings_scope import NotificationSettingsScope
from .option_value import OptionValue
from .order_info import OrderInfo
from .poll import Poll
from .reply_markup import ReplyMarkup
from .scope_notification_settings import ScopeNotificationSettings
from .secret_chat import SecretChat
from .sticker_set import StickerSet
from .sticker_sets import StickerSets
from .suggested_action import SuggestedAction
from .supergroup import Supergroup
from .supergroup_full_info import SupergroupFullInfo
from .terms_of_service import TermsOfService
from .user import User
from .user_full_info import UserFullInfo
from .user_privacy_setting import UserPrivacySetting
from .user_privacy_setting_rules import UserPrivacySettingRules
from .user_status import UserStatus
from .voice_chat import VoiceChat
from ..base_object import BaseObject


class Update(BaseObject):
    """
    Contains notifications about data changes
    
    """

    ID: str = Field("update", alias="@type")


class UpdateActiveNotifications(Update):
    """
    Contains active notifications that was shown on previous application launches. This update is sent only if the message database is used. In that case it comes once before any updateNotification and updateNotificationGroup update
    
    Params:
        groups (:obj:`list[NotificationGroup]`)
            Lists of active notification groups
        
    """

    ID: str = Field("updateActiveNotifications", alias="@type")
    groups: list[NotificationGroup]

    @staticmethod
    def read(q: dict) -> UpdateActiveNotifications:
        return UpdateActiveNotifications.construct(**q)


class UpdateAnimationSearchParameters(Update):
    """
    The parameters of animation search through GetOption("animation_search_bot_username") bot has changed
    
    Params:
        provider (:class:`str`)
            Name of the animation search provider
        
        emojis (:obj:`list[str]`)
            The new list of emojis suggested for searching
        
    """

    ID: str = Field("updateAnimationSearchParameters", alias="@type")
    provider: str
    emojis: list[str]

    @staticmethod
    def read(q: dict) -> UpdateAnimationSearchParameters:
        return UpdateAnimationSearchParameters.construct(**q)


class UpdateAuthorizationState(Update):
    """
    The user authorization state has changed
    
    Params:
        authorization_state (:class:`AuthorizationState`)
            New authorization state
        
    """

    ID: str = Field("updateAuthorizationState", alias="@type")
    authorization_state: AuthorizationState

    @staticmethod
    def read(q: dict) -> UpdateAuthorizationState:
        return UpdateAuthorizationState.construct(**q)


class UpdateBasicGroup(Update):
    """
    Some data of a basic group has changed. This update is guaranteed to come before the basic group identifier is returned to the application
    
    Params:
        basic_group (:class:`BasicGroup`)
            New data about the group
        
    """

    ID: str = Field("updateBasicGroup", alias="@type")
    basic_group: BasicGroup

    @staticmethod
    def read(q: dict) -> UpdateBasicGroup:
        return UpdateBasicGroup.construct(**q)


class UpdateBasicGroupFullInfo(Update):
    """
    Some data from basicGroupFullInfo has been changed
    
    Params:
        basic_group_id (:class:`int`)
            Identifier of a basic group
        
        basic_group_full_info (:class:`BasicGroupFullInfo`)
            New full information about the group
        
    """

    ID: str = Field("updateBasicGroupFullInfo", alias="@type")
    basic_group_id: int
    basic_group_full_info: BasicGroupFullInfo

    @staticmethod
    def read(q: dict) -> UpdateBasicGroupFullInfo:
        return UpdateBasicGroupFullInfo.construct(**q)


class UpdateCall(Update):
    """
    New call was created or information about a call was updated
    
    Params:
        call (:class:`Call`)
            New data about a call
        
    """

    ID: str = Field("updateCall", alias="@type")
    call: Call

    @staticmethod
    def read(q: dict) -> UpdateCall:
        return UpdateCall.construct(**q)


class UpdateChatActionBar(Update):
    """
    The chat action bar was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        action_bar (:class:`ChatActionBar`)
            The new value of the action bar; may be null
        
    """

    ID: str = Field("updateChatActionBar", alias="@type")
    chat_id: int
    action_bar: typing.Optional[ChatActionBar] = None

    @staticmethod
    def read(q: dict) -> UpdateChatActionBar:
        return UpdateChatActionBar.construct(**q)


class UpdateChatDefaultDisableNotification(Update):
    """
    The value of the default disable_notification parameter, used when a message is sent to the chat, was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        default_disable_notification (:class:`bool`)
            The new default_disable_notification value
        
    """

    ID: str = Field("updateChatDefaultDisableNotification", alias="@type")
    chat_id: int
    default_disable_notification: bool

    @staticmethod
    def read(q: dict) -> UpdateChatDefaultDisableNotification:
        return UpdateChatDefaultDisableNotification.construct(**q)


class UpdateChatDraftMessage(Update):
    """
    A chat draft has changed. Be aware that the update may come in the currently opened chat but with old content of the draft. If the user has changed the content of the draft, this update shouldn't be applied
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        draft_message (:class:`DraftMessage`)
            The new draft message; may be null
        
        positions (:obj:`list[ChatPosition]`)
            The new chat positions in the chat lists
        
    """

    ID: str = Field("updateChatDraftMessage", alias="@type")
    chat_id: int
    draft_message: typing.Optional[DraftMessage] = None
    positions: list[ChatPosition]

    @staticmethod
    def read(q: dict) -> UpdateChatDraftMessage:
        return UpdateChatDraftMessage.construct(**q)


class UpdateChatFilters(Update):
    """
    The list of chat filters or a chat filter has changed
    
    Params:
        chat_filters (:obj:`list[ChatFilterInfo]`)
            The new list of chat filters
        
    """

    ID: str = Field("updateChatFilters", alias="@type")
    chat_filters: list[ChatFilterInfo]

    @staticmethod
    def read(q: dict) -> UpdateChatFilters:
        return UpdateChatFilters.construct(**q)


class UpdateChatHasScheduledMessages(Update):
    """
    A chat's has_scheduled_messages field has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        has_scheduled_messages (:class:`bool`)
            New value of has_scheduled_messages
        
    """

    ID: str = Field("updateChatHasScheduledMessages", alias="@type")
    chat_id: int
    has_scheduled_messages: bool

    @staticmethod
    def read(q: dict) -> UpdateChatHasScheduledMessages:
        return UpdateChatHasScheduledMessages.construct(**q)


class UpdateChatIsBlocked(Update):
    """
    A chat was blocked or unblocked
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        is_blocked (:class:`bool`)
            New value of is_blocked
        
    """

    ID: str = Field("updateChatIsBlocked", alias="@type")
    chat_id: int
    is_blocked: bool

    @staticmethod
    def read(q: dict) -> UpdateChatIsBlocked:
        return UpdateChatIsBlocked.construct(**q)


class UpdateChatIsMarkedAsUnread(Update):
    """
    A chat was marked as unread or was read
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        is_marked_as_unread (:class:`bool`)
            New value of is_marked_as_unread
        
    """

    ID: str = Field("updateChatIsMarkedAsUnread", alias="@type")
    chat_id: int
    is_marked_as_unread: bool

    @staticmethod
    def read(q: dict) -> UpdateChatIsMarkedAsUnread:
        return UpdateChatIsMarkedAsUnread.construct(**q)


class UpdateChatLastMessage(Update):
    """
    The last message of a chat was changed. If last_message is null, then the last message in the chat became unknown. Some new unknown messages might be added to the chat in this case
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        last_message (:class:`Message`)
            The new last message in the chat; may be null
        
        positions (:obj:`list[ChatPosition]`)
            The new chat positions in the chat lists
        
    """

    ID: str = Field("updateChatLastMessage", alias="@type")
    chat_id: int
    last_message: typing.Optional[Message] = None
    positions: list[ChatPosition]

    @staticmethod
    def read(q: dict) -> UpdateChatLastMessage:
        return UpdateChatLastMessage.construct(**q)


class UpdateChatMember(Update):
    """
    User rights changed in a chat; for bots only
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        actor_user_id (:class:`int`)
            Identifier of the user, changing the rights
        
        date (:class:`int`)
            Point in time (Unix timestamp) when the user rights was changed
        
        invite_link (:class:`ChatInviteLink`)
            If user has joined the chat using an invite link, the invite link; may be null
        
        old_chat_member (:class:`ChatMember`)
            Previous chat member
        
        new_chat_member (:class:`ChatMember`)
            New chat member
        
    """

    ID: str = Field("updateChatMember", alias="@type")
    chat_id: int
    actor_user_id: int
    date: int
    invite_link: typing.Optional[ChatInviteLink] = None
    old_chat_member: ChatMember
    new_chat_member: ChatMember

    @staticmethod
    def read(q: dict) -> UpdateChatMember:
        return UpdateChatMember.construct(**q)


class UpdateChatMessageTtlSetting(Update):
    """
    The message Time To Live setting for a chat was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_ttl_setting (:class:`int`)
            New value of message_ttl_setting
        
    """

    ID: str = Field("updateChatMessageTtlSetting", alias="@type")
    chat_id: int
    message_ttl_setting: int

    @staticmethod
    def read(q: dict) -> UpdateChatMessageTtlSetting:
        return UpdateChatMessageTtlSetting.construct(**q)


class UpdateChatNotificationSettings(Update):
    """
    Notification settings for a chat were changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        notification_settings (:class:`ChatNotificationSettings`)
            The new notification settings
        
    """

    ID: str = Field("updateChatNotificationSettings", alias="@type")
    chat_id: int
    notification_settings: ChatNotificationSettings

    @staticmethod
    def read(q: dict) -> UpdateChatNotificationSettings:
        return UpdateChatNotificationSettings.construct(**q)


class UpdateChatOnlineMemberCount(Update):
    """
    The number of online group members has changed. This update with non-zero count is sent only for currently opened chats. There is no guarantee that it will be sent just after the count has changed
    
    Params:
        chat_id (:class:`int`)
            Identifier of the chat
        
        online_member_count (:class:`int`)
            New number of online members in the chat, or 0 if unknown
        
    """

    ID: str = Field("updateChatOnlineMemberCount", alias="@type")
    chat_id: int
    online_member_count: int

    @staticmethod
    def read(q: dict) -> UpdateChatOnlineMemberCount:
        return UpdateChatOnlineMemberCount.construct(**q)


class UpdateChatPermissions(Update):
    """
    Chat permissions was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        permissions (:class:`ChatPermissions`)
            The new chat permissions
        
    """

    ID: str = Field("updateChatPermissions", alias="@type")
    chat_id: int
    permissions: ChatPermissions

    @staticmethod
    def read(q: dict) -> UpdateChatPermissions:
        return UpdateChatPermissions.construct(**q)


class UpdateChatPhoto(Update):
    """
    A chat photo was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        photo (:class:`ChatPhotoInfo`)
            The new chat photo; may be null
        
    """

    ID: str = Field("updateChatPhoto", alias="@type")
    chat_id: int
    photo: typing.Optional[ChatPhotoInfo] = None

    @staticmethod
    def read(q: dict) -> UpdateChatPhoto:
        return UpdateChatPhoto.construct(**q)


class UpdateChatPosition(Update):
    """
    The position of a chat in a chat list has changed. Instead of this update updateChatLastMessage or updateChatDraftMessage might be sent
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        position (:class:`ChatPosition`)
            New chat position. If new order is 0, then the chat needs to be removed from the list
        
    """

    ID: str = Field("updateChatPosition", alias="@type")
    chat_id: int
    position: ChatPosition

    @staticmethod
    def read(q: dict) -> UpdateChatPosition:
        return UpdateChatPosition.construct(**q)


class UpdateChatReadInbox(Update):
    """
    Incoming messages were read or number of unread messages has been changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        last_read_inbox_message_id (:class:`int`)
            Identifier of the last read incoming message
        
        unread_count (:class:`int`)
            The number of unread messages left in the chat
        
    """

    ID: str = Field("updateChatReadInbox", alias="@type")
    chat_id: int
    last_read_inbox_message_id: int
    unread_count: int

    @staticmethod
    def read(q: dict) -> UpdateChatReadInbox:
        return UpdateChatReadInbox.construct(**q)


class UpdateChatReadOutbox(Update):
    """
    Outgoing messages were read
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        last_read_outbox_message_id (:class:`int`)
            Identifier of last read outgoing message
        
    """

    ID: str = Field("updateChatReadOutbox", alias="@type")
    chat_id: int
    last_read_outbox_message_id: int

    @staticmethod
    def read(q: dict) -> UpdateChatReadOutbox:
        return UpdateChatReadOutbox.construct(**q)


class UpdateChatReplyMarkup(Update):
    """
    The default chat reply markup was changed. Can occur because new messages with reply markup were received or because an old reply markup was hidden by the user
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        reply_markup_message_id (:class:`int`)
            Identifier of the message from which reply markup needs to be used; 0 if there is no default custom reply markup in the chat
        
    """

    ID: str = Field("updateChatReplyMarkup", alias="@type")
    chat_id: int
    reply_markup_message_id: int

    @staticmethod
    def read(q: dict) -> UpdateChatReplyMarkup:
        return UpdateChatReplyMarkup.construct(**q)


class UpdateChatTitle(Update):
    """
    The title of a chat was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        title (:class:`str`)
            The new chat title
        
    """

    ID: str = Field("updateChatTitle", alias="@type")
    chat_id: int
    title: str

    @staticmethod
    def read(q: dict) -> UpdateChatTitle:
        return UpdateChatTitle.construct(**q)


class UpdateChatUnreadMentionCount(Update):
    """
    The chat unread_mention_count has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        unread_mention_count (:class:`int`)
            The number of unread mention messages left in the chat
        
    """

    ID: str = Field("updateChatUnreadMentionCount", alias="@type")
    chat_id: int
    unread_mention_count: int

    @staticmethod
    def read(q: dict) -> UpdateChatUnreadMentionCount:
        return UpdateChatUnreadMentionCount.construct(**q)


class UpdateChatVoiceChat(Update):
    """
    A chat voice chat state has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        voice_chat (:class:`VoiceChat`)
            New value of voice_chat
        
    """

    ID: str = Field("updateChatVoiceChat", alias="@type")
    chat_id: int
    voice_chat: VoiceChat

    @staticmethod
    def read(q: dict) -> UpdateChatVoiceChat:
        return UpdateChatVoiceChat.construct(**q)


class UpdateConnectionState(Update):
    """
    The connection state has changed. This update must be used only to show a human-readable description of the connection state
    
    Params:
        state (:class:`ConnectionState`)
            The new connection state
        
    """

    ID: str = Field("updateConnectionState", alias="@type")
    state: ConnectionState

    @staticmethod
    def read(q: dict) -> UpdateConnectionState:
        return UpdateConnectionState.construct(**q)


class UpdateDeleteMessages(Update):
    """
    Some messages were deleted
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_ids (:obj:`list[int]`)
            Identifiers of the deleted messages
        
        is_permanent (:class:`bool`)
            True, if the messages are permanently deleted by a user (as opposed to just becoming inaccessible)
        
        from_cache (:class:`bool`)
            True, if the messages are deleted only from the cache and can possibly be retrieved again in the future
        
    """

    ID: str = Field("updateDeleteMessages", alias="@type")
    chat_id: int
    message_ids: list[int]
    is_permanent: bool
    from_cache: bool

    @staticmethod
    def read(q: dict) -> UpdateDeleteMessages:
        return UpdateDeleteMessages.construct(**q)


class UpdateDiceEmojis(Update):
    """
    The list of supported dice emojis has changed
    
    Params:
        emojis (:obj:`list[str]`)
            The new list of supported dice emojis
        
    """

    ID: str = Field("updateDiceEmojis", alias="@type")
    emojis: list[str]

    @staticmethod
    def read(q: dict) -> UpdateDiceEmojis:
        return UpdateDiceEmojis.construct(**q)


class UpdateFavoriteStickers(Update):
    """
    The list of favorite stickers was updated
    
    Params:
        sticker_ids (:obj:`list[int]`)
            The new list of file identifiers of favorite stickers
        
    """

    ID: str = Field("updateFavoriteStickers", alias="@type")
    sticker_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdateFavoriteStickers:
        return UpdateFavoriteStickers.construct(**q)


class UpdateFile(Update):
    """
    Information about a file was updated
    
    Params:
        file (:class:`File`)
            New data about the file
        
    """

    ID: str = Field("updateFile", alias="@type")
    file: File

    @staticmethod
    def read(q: dict) -> UpdateFile:
        return UpdateFile.construct(**q)


class UpdateFileGenerationStart(Update):
    """
    The file generation process needs to be started by the application
    
    Params:
        generation_id (:class:`int`)
            Unique identifier for the generation process
        
        original_path (:class:`str`)
            The path to a file from which a new file is generated; may be empty
        
        destination_path (:class:`str`)
            The path to a file that should be created and where the new file should be generated
        
        conversion (:class:`str`)
            String specifying the conversion applied to the original file. If conversion is "#url#" than original_path contains an HTTP/HTTPS URL of a file, which should be downloaded by the application
        
    """

    ID: str = Field("updateFileGenerationStart", alias="@type")
    generation_id: int
    original_path: str
    destination_path: str
    conversion: str

    @staticmethod
    def read(q: dict) -> UpdateFileGenerationStart:
        return UpdateFileGenerationStart.construct(**q)


class UpdateFileGenerationStop(Update):
    """
    File generation is no longer needed
    
    Params:
        generation_id (:class:`int`)
            Unique identifier for the generation process
        
    """

    ID: str = Field("updateFileGenerationStop", alias="@type")
    generation_id: int

    @staticmethod
    def read(q: dict) -> UpdateFileGenerationStop:
        return UpdateFileGenerationStop.construct(**q)


class UpdateGroupCall(Update):
    """
    Information about a group call was updated
    
    Params:
        group_call (:class:`GroupCall`)
            New data about a group call
        
    """

    ID: str = Field("updateGroupCall", alias="@type")
    group_call: GroupCall

    @staticmethod
    def read(q: dict) -> UpdateGroupCall:
        return UpdateGroupCall.construct(**q)


class UpdateGroupCallParticipant(Update):
    """
    Information about a group call participant was changed. The updates are sent only after the group call is received through getGroupCall and only if the call is joined or being joined
    
    Params:
        group_call_id (:class:`int`)
            Identifier of group call
        
        participant (:class:`GroupCallParticipant`)
            New data about a participant
        
    """

    ID: str = Field("updateGroupCallParticipant", alias="@type")
    group_call_id: int
    participant: GroupCallParticipant

    @staticmethod
    def read(q: dict) -> UpdateGroupCallParticipant:
        return UpdateGroupCallParticipant.construct(**q)


class UpdateHavePendingNotifications(Update):
    """
    Describes whether there are some pending notification updates. Can be used to prevent application from killing, while there are some pending notifications
    
    Params:
        have_delayed_notifications (:class:`bool`)
            True, if there are some delayed notification updates, which will be sent soon
        
        have_unreceived_notifications (:class:`bool`)
            True, if there can be some yet unreceived notifications, which are being fetched from the server
        
    """

    ID: str = Field("updateHavePendingNotifications", alias="@type")
    have_delayed_notifications: bool
    have_unreceived_notifications: bool

    @staticmethod
    def read(q: dict) -> UpdateHavePendingNotifications:
        return UpdateHavePendingNotifications.construct(**q)


class UpdateInstalledStickerSets(Update):
    """
    The list of installed sticker sets was updated
    
    Params:
        is_masks (:class:`bool`)
            True, if the list of installed mask sticker sets was updated
        
        sticker_set_ids (:obj:`list[int]`)
            The new list of installed ordinary sticker sets
        
    """

    ID: str = Field("updateInstalledStickerSets", alias="@type")
    is_masks: bool
    sticker_set_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdateInstalledStickerSets:
        return UpdateInstalledStickerSets.construct(**q)


class UpdateLanguagePackStrings(Update):
    """
    Some language pack strings have been updated
    
    Params:
        localization_target (:class:`str`)
            Localization target to which the language pack belongs
        
        language_pack_id (:class:`str`)
            Identifier of the updated language pack
        
        strings (:obj:`list[LanguagePackString]`)
            List of changed language pack strings
        
    """

    ID: str = Field("updateLanguagePackStrings", alias="@type")
    localization_target: str
    language_pack_id: str
    strings: list[LanguagePackString]

    @staticmethod
    def read(q: dict) -> UpdateLanguagePackStrings:
        return UpdateLanguagePackStrings.construct(**q)


class UpdateMessageContent(Update):
    """
    The message content has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            Message identifier
        
        new_content (:class:`MessageContent`)
            New message content
        
    """

    ID: str = Field("updateMessageContent", alias="@type")
    chat_id: int
    message_id: int
    new_content: MessageContent

    @staticmethod
    def read(q: dict) -> UpdateMessageContent:
        return UpdateMessageContent.construct(**q)


class UpdateMessageContentOpened(Update):
    """
    The message content was opened. Updates voice note messages to "listened", video note messages to "viewed" and starts the TTL timer for self-destructing messages
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            Message identifier
        
    """

    ID: str = Field("updateMessageContentOpened", alias="@type")
    chat_id: int
    message_id: int

    @staticmethod
    def read(q: dict) -> UpdateMessageContentOpened:
        return UpdateMessageContentOpened.construct(**q)


class UpdateMessageEdited(Update):
    """
    A message was edited. Changes in the message content will come in a separate updateMessageContent
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            Message identifier
        
        edit_date (:class:`int`)
            Point in time (Unix timestamp) when the message was edited
        
        reply_markup (:class:`ReplyMarkup`)
            New message reply markup; may be null
        
    """

    ID: str = Field("updateMessageEdited", alias="@type")
    chat_id: int
    message_id: int
    edit_date: int
    reply_markup: typing.Optional[ReplyMarkup] = None

    @staticmethod
    def read(q: dict) -> UpdateMessageEdited:
        return UpdateMessageEdited.construct(**q)


class UpdateMessageInteractionInfo(Update):
    """
    The information about interactions with a message has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            Message identifier
        
        interaction_info (:class:`MessageInteractionInfo`)
            New information about interactions with the message; may be null
        
    """

    ID: str = Field("updateMessageInteractionInfo", alias="@type")
    chat_id: int
    message_id: int
    interaction_info: typing.Optional[MessageInteractionInfo] = None

    @staticmethod
    def read(q: dict) -> UpdateMessageInteractionInfo:
        return UpdateMessageInteractionInfo.construct(**q)


class UpdateMessageIsPinned(Update):
    """
    The message pinned state was changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            The message identifier
        
        is_pinned (:class:`bool`)
            True, if the message is pinned
        
    """

    ID: str = Field("updateMessageIsPinned", alias="@type")
    chat_id: int
    message_id: int
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> UpdateMessageIsPinned:
        return UpdateMessageIsPinned.construct(**q)


class UpdateMessageLiveLocationViewed(Update):
    """
    A message with a live location was viewed. When the update is received, the application is supposed to update the live location
    
    Params:
        chat_id (:class:`int`)
            Identifier of the chat with the live location message
        
        message_id (:class:`int`)
            Identifier of the message with live location
        
    """

    ID: str = Field("updateMessageLiveLocationViewed", alias="@type")
    chat_id: int
    message_id: int

    @staticmethod
    def read(q: dict) -> UpdateMessageLiveLocationViewed:
        return UpdateMessageLiveLocationViewed.construct(**q)


class UpdateMessageMentionRead(Update):
    """
    A message with an unread mention was read
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_id (:class:`int`)
            Message identifier
        
        unread_mention_count (:class:`int`)
            The new number of unread mention messages left in the chat
        
    """

    ID: str = Field("updateMessageMentionRead", alias="@type")
    chat_id: int
    message_id: int
    unread_mention_count: int

    @staticmethod
    def read(q: dict) -> UpdateMessageMentionRead:
        return UpdateMessageMentionRead.construct(**q)


class UpdateMessageSendAcknowledged(Update):
    """
    A request to send a message has reached the Telegram server. This doesn't mean that the message will be sent successfully or even that the send message request will be processed. This update will be sent only if the option "use_quick_ack" is set to true. This update may be sent multiple times for the same message
    
    Params:
        chat_id (:class:`int`)
            The chat identifier of the sent message
        
        message_id (:class:`int`)
            A temporary message identifier
        
    """

    ID: str = Field("updateMessageSendAcknowledged", alias="@type")
    chat_id: int
    message_id: int

    @staticmethod
    def read(q: dict) -> UpdateMessageSendAcknowledged:
        return UpdateMessageSendAcknowledged.construct(**q)


class UpdateMessageSendFailed(Update):
    """
    A message failed to send. Be aware that some messages being sent can be irrecoverably deleted, in which case updateDeleteMessages will be received instead of this update
    
    Params:
        message (:class:`Message`)
            Contains information about the message which failed to send
        
        old_message_id (:class:`int`)
            The previous temporary message identifier
        
        error_code (:class:`int`)
            An error code
        
        error_message (:class:`str`)
            Error message
        
    """

    ID: str = Field("updateMessageSendFailed", alias="@type")
    message: Message
    old_message_id: int
    error_code: int
    error_message: str

    @staticmethod
    def read(q: dict) -> UpdateMessageSendFailed:
        return UpdateMessageSendFailed.construct(**q)


class UpdateMessageSendSucceeded(Update):
    """
    A message has been successfully sent
    
    Params:
        message (:class:`Message`)
            Information about the sent message. Usually only the message identifier, date, and content are changed, but almost all other fields can also change
        
        old_message_id (:class:`int`)
            The previous temporary message identifier
        
    """

    ID: str = Field("updateMessageSendSucceeded", alias="@type")
    message: Message
    old_message_id: int

    @staticmethod
    def read(q: dict) -> UpdateMessageSendSucceeded:
        return UpdateMessageSendSucceeded.construct(**q)


class UpdateNewCallSignalingData(Update):
    """
    New call signaling data arrived
    
    Params:
        call_id (:class:`int`)
            The call identifier
        
        data (:class:`str`)
            The data
        
    """

    ID: str = Field("updateNewCallSignalingData", alias="@type")
    call_id: int
    data: str

    @staticmethod
    def read(q: dict) -> UpdateNewCallSignalingData:
        return UpdateNewCallSignalingData.construct(**q)


class UpdateNewCallbackQuery(Update):
    """
    A new incoming callback query; for bots only
    
    Params:
        id (:class:`int`)
            Unique query identifier
        
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        chat_id (:class:`int`)
            Identifier of the chat where the query was sent
        
        message_id (:class:`int`)
            Identifier of the message, from which the query originated
        
        chat_instance (:class:`int`)
            Identifier that uniquely corresponds to the chat to which the message was sent
        
        payload (:class:`CallbackQueryPayload`)
            Query payload
        
    """

    ID: str = Field("updateNewCallbackQuery", alias="@type")
    id: int
    sender_user_id: int
    chat_id: int
    message_id: int
    chat_instance: int
    payload: CallbackQueryPayload

    @staticmethod
    def read(q: dict) -> UpdateNewCallbackQuery:
        return UpdateNewCallbackQuery.construct(**q)


class UpdateNewChat(Update):
    """
    A new chat has been loaded/created. This update is guaranteed to come before the chat identifier is returned to the application. The chat field changes will be reported through separate updates
    
    Params:
        chat (:class:`Chat`)
            The chat
        
    """

    ID: str = Field("updateNewChat", alias="@type")
    chat: Chat

    @staticmethod
    def read(q: dict) -> UpdateNewChat:
        return UpdateNewChat.construct(**q)


class UpdateNewChosenInlineResult(Update):
    """
    The user has chosen a result of an inline query; for bots only
    
    Params:
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        user_location (:class:`Location`)
            User location; may be null
        
        query (:class:`str`)
            Text of the query
        
        result_id (:class:`str`)
            Identifier of the chosen result
        
        inline_message_id (:class:`str`)
            Identifier of the sent inline message, if known
        
    """

    ID: str = Field("updateNewChosenInlineResult", alias="@type")
    sender_user_id: int
    user_location: typing.Optional[Location] = None
    query: str
    result_id: str
    inline_message_id: str

    @staticmethod
    def read(q: dict) -> UpdateNewChosenInlineResult:
        return UpdateNewChosenInlineResult.construct(**q)


class UpdateNewCustomEvent(Update):
    """
    A new incoming event; for bots only
    
    Params:
        event (:class:`str`)
            A JSON-serialized event
        
    """

    ID: str = Field("updateNewCustomEvent", alias="@type")
    event: str

    @staticmethod
    def read(q: dict) -> UpdateNewCustomEvent:
        return UpdateNewCustomEvent.construct(**q)


class UpdateNewCustomQuery(Update):
    """
    A new incoming query; for bots only
    
    Params:
        id (:class:`int`)
            The query identifier
        
        data (:class:`str`)
            JSON-serialized query data
        
        timeout (:class:`int`)
            Query timeout
        
    """

    ID: str = Field("updateNewCustomQuery", alias="@type")
    id: int
    data: str
    timeout: int

    @staticmethod
    def read(q: dict) -> UpdateNewCustomQuery:
        return UpdateNewCustomQuery.construct(**q)


class UpdateNewInlineCallbackQuery(Update):
    """
    A new incoming callback query from a message sent via a bot; for bots only
    
    Params:
        id (:class:`int`)
            Unique query identifier
        
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        inline_message_id (:class:`str`)
            Identifier of the inline message, from which the query originated
        
        chat_instance (:class:`int`)
            An identifier uniquely corresponding to the chat a message was sent to
        
        payload (:class:`CallbackQueryPayload`)
            Query payload
        
    """

    ID: str = Field("updateNewInlineCallbackQuery", alias="@type")
    id: int
    sender_user_id: int
    inline_message_id: str
    chat_instance: int
    payload: CallbackQueryPayload

    @staticmethod
    def read(q: dict) -> UpdateNewInlineCallbackQuery:
        return UpdateNewInlineCallbackQuery.construct(**q)


class UpdateNewInlineQuery(Update):
    """
    A new incoming inline query; for bots only
    
    Params:
        id (:class:`int`)
            Unique query identifier
        
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        user_location (:class:`Location`)
            User location; may be null
        
        chat_type (:class:`ChatType`)
            Contains information about the type of the chat, from which the query originated; may be null if unknown
        
        query (:class:`str`)
            Text of the query
        
        offset (:class:`str`)
            Offset of the first entry to return
        
    """

    ID: str = Field("updateNewInlineQuery", alias="@type")
    id: int
    sender_user_id: int
    user_location: typing.Optional[Location] = None
    chat_type: typing.Optional[ChatType] = None
    query: str
    offset: str

    @staticmethod
    def read(q: dict) -> UpdateNewInlineQuery:
        return UpdateNewInlineQuery.construct(**q)


class UpdateNewMessage(Update):
    """
    A new message was received; can also be an outgoing message
    
    Params:
        message (:class:`Message`)
            The new message
        
    """

    ID: str = Field("updateNewMessage", alias="@type")
    message: Message

    @staticmethod
    def read(q: dict) -> UpdateNewMessage:
        return UpdateNewMessage.construct(**q)


class UpdateNewPreCheckoutQuery(Update):
    """
    A new incoming pre-checkout query; for bots only. Contains full information about a checkout
    
    Params:
        id (:class:`int`)
            Unique query identifier
        
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        currency (:class:`str`)
            Currency for the product price
        
        total_amount (:class:`int`)
            Total price for the product, in the smallest units of the currency
        
        invoice_payload (:class:`str`)
            Invoice payload
        
        shipping_option_id (:class:`str`)
            Identifier of a shipping option chosen by the user; may be empty if not applicable
        
        order_info (:class:`OrderInfo`)
            Information about the order; may be null
        
    """

    ID: str = Field("updateNewPreCheckoutQuery", alias="@type")
    id: int
    sender_user_id: int
    currency: str
    total_amount: int
    invoice_payload: str
    shipping_option_id: str
    order_info: typing.Optional[OrderInfo] = None

    @staticmethod
    def read(q: dict) -> UpdateNewPreCheckoutQuery:
        return UpdateNewPreCheckoutQuery.construct(**q)


class UpdateNewShippingQuery(Update):
    """
    A new incoming shipping query; for bots only. Only for invoices with flexible price
    
    Params:
        id (:class:`int`)
            Unique query identifier
        
        sender_user_id (:class:`int`)
            Identifier of the user who sent the query
        
        invoice_payload (:class:`str`)
            Invoice payload
        
        shipping_address (:class:`Address`)
            User shipping address
        
    """

    ID: str = Field("updateNewShippingQuery", alias="@type")
    id: int
    sender_user_id: int
    invoice_payload: str
    shipping_address: Address

    @staticmethod
    def read(q: dict) -> UpdateNewShippingQuery:
        return UpdateNewShippingQuery.construct(**q)


class UpdateNotification(Update):
    """
    A notification was changed
    
    Params:
        notification_group_id (:class:`int`)
            Unique notification group identifier
        
        notification (:class:`Notification`)
            Changed notification
        
    """

    ID: str = Field("updateNotification", alias="@type")
    notification_group_id: int
    notification: Notification

    @staticmethod
    def read(q: dict) -> UpdateNotification:
        return UpdateNotification.construct(**q)


class UpdateNotificationGroup(Update):
    """
    A list of active notifications in a notification group has changed
    
    Params:
        notification_group_id (:class:`int`)
            Unique notification group identifier
        
        type_ (:class:`NotificationGroupType`)
            New type of the notification group
        
        chat_id (:class:`int`)
            Identifier of a chat to which all notifications in the group belong
        
        notification_settings_chat_id (:class:`int`)
            Chat identifier, which notification settings must be applied to the added notifications
        
        is_silent (:class:`bool`)
            True, if the notifications should be shown without sound
        
        total_count (:class:`int`)
            Total number of unread notifications in the group, can be bigger than number of active notifications
        
        added_notifications (:obj:`list[Notification]`)
            List of added group notifications, sorted by notification ID
        
        removed_notification_ids (:obj:`list[int]`)
            Identifiers of removed group notifications, sorted by notification ID
        
    """

    ID: str = Field("updateNotificationGroup", alias="@type")
    notification_group_id: int
    type_: NotificationGroupType = Field(..., alias='type')
    chat_id: int
    notification_settings_chat_id: int
    is_silent: bool
    total_count: int
    added_notifications: list[Notification]
    removed_notification_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdateNotificationGroup:
        return UpdateNotificationGroup.construct(**q)


class UpdateOption(Update):
    """
    An option changed its value
    
    Params:
        name (:class:`str`)
            The option name
        
        value (:class:`OptionValue`)
            The new option value
        
    """

    ID: str = Field("updateOption", alias="@type")
    name: str
    value: OptionValue

    @staticmethod
    def read(q: dict) -> UpdateOption:
        return UpdateOption.construct(**q)


class UpdatePoll(Update):
    """
    A poll was updated; for bots only
    
    Params:
        poll (:class:`Poll`)
            New data about the poll
        
    """

    ID: str = Field("updatePoll", alias="@type")
    poll: Poll

    @staticmethod
    def read(q: dict) -> UpdatePoll:
        return UpdatePoll.construct(**q)


class UpdatePollAnswer(Update):
    """
    A user changed the answer to a poll; for bots only
    
    Params:
        poll_id (:class:`int`)
            Unique poll identifier
        
        user_id (:class:`int`)
            The user, who changed the answer to the poll
        
        option_ids (:obj:`list[int]`)
            0-based identifiers of answer options, chosen by the user
        
    """

    ID: str = Field("updatePollAnswer", alias="@type")
    poll_id: int
    user_id: int
    option_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdatePollAnswer:
        return UpdatePollAnswer.construct(**q)


class UpdateRecentStickers(Update):
    """
    The list of recently used stickers was updated
    
    Params:
        is_attached (:class:`bool`)
            True, if the list of stickers attached to photo or video files was updated, otherwise the list of sent stickers is updated
        
        sticker_ids (:obj:`list[int]`)
            The new list of file identifiers of recently used stickers
        
    """

    ID: str = Field("updateRecentStickers", alias="@type")
    is_attached: bool
    sticker_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdateRecentStickers:
        return UpdateRecentStickers.construct(**q)


class UpdateSavedAnimations(Update):
    """
    The list of saved animations was updated
    
    Params:
        animation_ids (:obj:`list[int]`)
            The new list of file identifiers of saved animations
        
    """

    ID: str = Field("updateSavedAnimations", alias="@type")
    animation_ids: list[int]

    @staticmethod
    def read(q: dict) -> UpdateSavedAnimations:
        return UpdateSavedAnimations.construct(**q)


class UpdateScopeNotificationSettings(Update):
    """
    Notification settings for some type of chats were updated
    
    Params:
        scope (:class:`NotificationSettingsScope`)
            Types of chats for which notification settings were updated
        
        notification_settings (:class:`ScopeNotificationSettings`)
            The new notification settings
        
    """

    ID: str = Field("updateScopeNotificationSettings", alias="@type")
    scope: NotificationSettingsScope
    notification_settings: ScopeNotificationSettings

    @staticmethod
    def read(q: dict) -> UpdateScopeNotificationSettings:
        return UpdateScopeNotificationSettings.construct(**q)


class UpdateSecretChat(Update):
    """
    Some data of a secret chat has changed. This update is guaranteed to come before the secret chat identifier is returned to the application
    
    Params:
        secret_chat (:class:`SecretChat`)
            New data about the secret chat
        
    """

    ID: str = Field("updateSecretChat", alias="@type")
    secret_chat: SecretChat

    @staticmethod
    def read(q: dict) -> UpdateSecretChat:
        return UpdateSecretChat.construct(**q)


class UpdateSelectedBackground(Update):
    """
    The selected background has changed
    
    Params:
        for_dark_theme (:class:`bool`)
            True, if background for dark theme has changed
        
        background (:class:`Background`)
            The new selected background; may be null
        
    """

    ID: str = Field("updateSelectedBackground", alias="@type")
    for_dark_theme: bool
    background: typing.Optional[Background] = None

    @staticmethod
    def read(q: dict) -> UpdateSelectedBackground:
        return UpdateSelectedBackground.construct(**q)


class UpdateServiceNotification(Update):
    """
    Service notification from the server. Upon receiving this the application must show a popup with the content of the notification
    
    Params:
        type_ (:class:`str`)
            Notification type. If type begins with "AUTH_KEY_DROP_", then two buttons "Cancel" and "Log out" should be shown under notification; if user presses the second, all local data should be destroyed using Destroy method
        
        content (:class:`MessageContent`)
            Notification content
        
    """

    ID: str = Field("updateServiceNotification", alias="@type")
    type_: str = Field(..., alias='type')
    content: MessageContent

    @staticmethod
    def read(q: dict) -> UpdateServiceNotification:
        return UpdateServiceNotification.construct(**q)


class UpdateStickerSet(Update):
    """
    A sticker set has changed
    
    Params:
        sticker_set (:class:`StickerSet`)
            The sticker set
        
    """

    ID: str = Field("updateStickerSet", alias="@type")
    sticker_set: StickerSet

    @staticmethod
    def read(q: dict) -> UpdateStickerSet:
        return UpdateStickerSet.construct(**q)


class UpdateSuggestedActions(Update):
    """
    The list of suggested to the user actions has changed
    
    Params:
        added_actions (:obj:`list[SuggestedAction]`)
            Added suggested actions
        
        removed_actions (:obj:`list[SuggestedAction]`)
            Removed suggested actions
        
    """

    ID: str = Field("updateSuggestedActions", alias="@type")
    added_actions: list[SuggestedAction]
    removed_actions: list[SuggestedAction]

    @staticmethod
    def read(q: dict) -> UpdateSuggestedActions:
        return UpdateSuggestedActions.construct(**q)


class UpdateSupergroup(Update):
    """
    Some data of a supergroup or a channel has changed. This update is guaranteed to come before the supergroup identifier is returned to the application
    
    Params:
        supergroup (:class:`Supergroup`)
            New data about the supergroup
        
    """

    ID: str = Field("updateSupergroup", alias="@type")
    supergroup: Supergroup

    @staticmethod
    def read(q: dict) -> UpdateSupergroup:
        return UpdateSupergroup.construct(**q)


class UpdateSupergroupFullInfo(Update):
    """
    Some data from supergroupFullInfo has been changed
    
    Params:
        supergroup_id (:class:`int`)
            Identifier of the supergroup or channel
        
        supergroup_full_info (:class:`SupergroupFullInfo`)
            New full information about the supergroup
        
    """

    ID: str = Field("updateSupergroupFullInfo", alias="@type")
    supergroup_id: int
    supergroup_full_info: SupergroupFullInfo

    @staticmethod
    def read(q: dict) -> UpdateSupergroupFullInfo:
        return UpdateSupergroupFullInfo.construct(**q)


class UpdateTermsOfService(Update):
    """
    New terms of service must be accepted by the user. If the terms of service are declined, then the deleteAccount method should be called with the reason "Decline ToS update"
    
    Params:
        terms_of_service_id (:class:`str`)
            Identifier of the terms of service
        
        terms_of_service (:class:`TermsOfService`)
            The new terms of service
        
    """

    ID: str = Field("updateTermsOfService", alias="@type")
    terms_of_service_id: str
    terms_of_service: TermsOfService

    @staticmethod
    def read(q: dict) -> UpdateTermsOfService:
        return UpdateTermsOfService.construct(**q)


class UpdateTrendingStickerSets(Update):
    """
    The list of trending sticker sets was updated or some of them were viewed
    
    Params:
        sticker_sets (:class:`StickerSets`)
            The prefix of the list of trending sticker sets with the newest trending sticker sets
        
    """

    ID: str = Field("updateTrendingStickerSets", alias="@type")
    sticker_sets: StickerSets

    @staticmethod
    def read(q: dict) -> UpdateTrendingStickerSets:
        return UpdateTrendingStickerSets.construct(**q)


class UpdateUnreadChatCount(Update):
    """
    Number of unread chats, i.e. with unread messages or marked as unread, has changed. This update is sent only if the message database is used
    
    Params:
        chat_list (:class:`ChatList`)
            The chat list with changed number of unread messages
        
        total_count (:class:`int`)
            Approximate total number of chats in the chat list
        
        unread_count (:class:`int`)
            Total number of unread chats
        
        unread_unmuted_count (:class:`int`)
            Total number of unread unmuted chats
        
        marked_as_unread_count (:class:`int`)
            Total number of chats marked as unread
        
        marked_as_unread_unmuted_count (:class:`int`)
            Total number of unmuted chats marked as unread
        
    """

    ID: str = Field("updateUnreadChatCount", alias="@type")
    chat_list: ChatList
    total_count: int
    unread_count: int
    unread_unmuted_count: int
    marked_as_unread_count: int
    marked_as_unread_unmuted_count: int

    @staticmethod
    def read(q: dict) -> UpdateUnreadChatCount:
        return UpdateUnreadChatCount.construct(**q)


class UpdateUnreadMessageCount(Update):
    """
    Number of unread messages in a chat list has changed. This update is sent only if the message database is used
    
    Params:
        chat_list (:class:`ChatList`)
            The chat list with changed number of unread messages
        
        unread_count (:class:`int`)
            Total number of unread messages
        
        unread_unmuted_count (:class:`int`)
            Total number of unread messages in unmuted chats
        
    """

    ID: str = Field("updateUnreadMessageCount", alias="@type")
    chat_list: ChatList
    unread_count: int
    unread_unmuted_count: int

    @staticmethod
    def read(q: dict) -> UpdateUnreadMessageCount:
        return UpdateUnreadMessageCount.construct(**q)


class UpdateUser(Update):
    """
    Some data of a user has changed. This update is guaranteed to come before the user identifier is returned to the application
    
    Params:
        user (:class:`User`)
            New data about the user
        
    """

    ID: str = Field("updateUser", alias="@type")
    user: User

    @staticmethod
    def read(q: dict) -> UpdateUser:
        return UpdateUser.construct(**q)


class UpdateUserChatAction(Update):
    """
    User activity in the chat has changed
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_thread_id (:class:`int`)
            If not 0, a message thread identifier in which the action was performed
        
        user_id (:class:`int`)
            Identifier of a user performing an action
        
        action (:class:`ChatAction`)
            The action description
        
    """

    ID: str = Field("updateUserChatAction", alias="@type")
    chat_id: int
    message_thread_id: int
    user_id: int
    action: ChatAction

    @staticmethod
    def read(q: dict) -> UpdateUserChatAction:
        return UpdateUserChatAction.construct(**q)


class UpdateUserFullInfo(Update):
    """
    Some data from userFullInfo has been changed
    
    Params:
        user_id (:class:`int`)
            User identifier
        
        user_full_info (:class:`UserFullInfo`)
            New full information about the user
        
    """

    ID: str = Field("updateUserFullInfo", alias="@type")
    user_id: int
    user_full_info: UserFullInfo

    @staticmethod
    def read(q: dict) -> UpdateUserFullInfo:
        return UpdateUserFullInfo.construct(**q)


class UpdateUserPrivacySettingRules(Update):
    """
    Some privacy setting rules have been changed
    
    Params:
        setting (:class:`UserPrivacySetting`)
            The privacy setting
        
        rules (:class:`UserPrivacySettingRules`)
            New privacy rules
        
    """

    ID: str = Field("updateUserPrivacySettingRules", alias="@type")
    setting: UserPrivacySetting
    rules: UserPrivacySettingRules

    @staticmethod
    def read(q: dict) -> UpdateUserPrivacySettingRules:
        return UpdateUserPrivacySettingRules.construct(**q)


class UpdateUserStatus(Update):
    """
    The user went online or offline
    
    Params:
        user_id (:class:`int`)
            User identifier
        
        status (:class:`UserStatus`)
            New status of the user
        
    """

    ID: str = Field("updateUserStatus", alias="@type")
    user_id: int
    status: UserStatus

    @staticmethod
    def read(q: dict) -> UpdateUserStatus:
        return UpdateUserStatus.construct(**q)


class UpdateUsersNearby(Update):
    """
    The list of users nearby has changed. The update is guaranteed to be sent only 60 seconds after a successful searchChatsNearby request
    
    Params:
        users_nearby (:obj:`list[ChatNearby]`)
            The new list of users nearby
        
    """

    ID: str = Field("updateUsersNearby", alias="@type")
    users_nearby: list[ChatNearby]

    @staticmethod
    def read(q: dict) -> UpdateUsersNearby:
        return UpdateUsersNearby.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class Hashtags(BaseObject):
    """
    Contains a list of hashtags
    
    Params:
        hashtags (:obj:`list[str]`)
            A list of hashtags
        
    """

    ID: str = Field("hashtags", alias="@type")
    hashtags: list[str]

    @staticmethod
    def read(q: dict) -> Hashtags:
        return Hashtags.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class RemoteFile(BaseObject):
    """
    Represents a remote file
    
    Params:
        id (:class:`str`)
            Remote file identifier; may be empty. Can be used by the current user across application restarts or even from other devices. Uniquely identifies a file, but a file can have a lot of different valid identifiers. If the ID starts with "http://" or "https://", it represents the HTTP URL of the file. TDLib is currently unable to download files if only their URL is known. If downloadFile is called on such a file or if it is sent to a secret chat, TDLib starts a file generation process by sending updateFileGenerationStart to the application with the HTTP URL in the original_path and "#url#" as the conversion string. Application should generate the file by downloading it to the specified location
        
        unique_id (:class:`str`)
            Unique file identifier; may be empty if unknown. The unique file identifier which is the same for the same file even for different users and is persistent over time
        
        is_uploading_active (:class:`bool`)
            True, if the file is currently being uploaded (or a remote copy is being generated by some other means)
        
        is_uploading_completed (:class:`bool`)
            True, if a remote copy is fully available
        
        uploaded_size (:class:`int`)
            Size of the remote available part of the file; 0 if unknown
        
    """

    ID: str = Field("remoteFile", alias="@type")
    id: str
    unique_id: str
    is_uploading_active: bool
    is_uploading_completed: bool
    uploaded_size: int

    @staticmethod
    def read(q: dict) -> RemoteFile:
        return RemoteFile.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SearchMessagesFilter(BaseObject):
    """
    Represents a filter for message search results
    
    """

    ID: str = Field("searchMessagesFilter", alias="@type")


class SearchMessagesFilterAnimation(SearchMessagesFilter):
    """
    Returns only animation messages
    
    """

    ID: str = Field("searchMessagesFilterAnimation", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterAnimation:
        return SearchMessagesFilterAnimation.construct(**q)


class SearchMessagesFilterAudio(SearchMessagesFilter):
    """
    Returns only audio messages
    
    """

    ID: str = Field("searchMessagesFilterAudio", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterAudio:
        return SearchMessagesFilterAudio.construct(**q)


class SearchMessagesFilterCall(SearchMessagesFilter):
    """
    Returns only call messages
    
    """

    ID: str = Field("searchMessagesFilterCall", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterCall:
        return SearchMessagesFilterCall.construct(**q)


class SearchMessagesFilterChatPhoto(SearchMessagesFilter):
    """
    Returns only messages containing chat photos
    
    """

    ID: str = Field("searchMessagesFilterChatPhoto", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterChatPhoto:
        return SearchMessagesFilterChatPhoto.construct(**q)


class SearchMessagesFilterDocument(SearchMessagesFilter):
    """
    Returns only document messages
    
    """

    ID: str = Field("searchMessagesFilterDocument", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterDocument:
        return SearchMessagesFilterDocument.construct(**q)


class SearchMessagesFilterEmpty(SearchMessagesFilter):
    """
    Returns all found messages, no filter is applied
    
    """

    ID: str = Field("searchMessagesFilterEmpty", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterEmpty:
        return SearchMessagesFilterEmpty.construct(**q)


class SearchMessagesFilterFailedToSend(SearchMessagesFilter):
    """
    Returns only failed to send messages. This filter can be used only if the message database is used
    
    """

    ID: str = Field("searchMessagesFilterFailedToSend", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterFailedToSend:
        return SearchMessagesFilterFailedToSend.construct(**q)


class SearchMessagesFilterMention(SearchMessagesFilter):
    """
    Returns only messages with mentions of the current user, or messages that are replies to their messages
    
    """

    ID: str = Field("searchMessagesFilterMention", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterMention:
        return SearchMessagesFilterMention.construct(**q)


class SearchMessagesFilterMissedCall(SearchMessagesFilter):
    """
    Returns only incoming call messages with missed/declined discard reasons
    
    """

    ID: str = Field("searchMessagesFilterMissedCall", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterMissedCall:
        return SearchMessagesFilterMissedCall.construct(**q)


class SearchMessagesFilterPhoto(SearchMessagesFilter):
    """
    Returns only photo messages
    
    """

    ID: str = Field("searchMessagesFilterPhoto", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterPhoto:
        return SearchMessagesFilterPhoto.construct(**q)


class SearchMessagesFilterPhotoAndVideo(SearchMessagesFilter):
    """
    Returns only photo and video messages
    
    """

    ID: str = Field("searchMessagesFilterPhotoAndVideo", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterPhotoAndVideo:
        return SearchMessagesFilterPhotoAndVideo.construct(**q)


class SearchMessagesFilterPinned(SearchMessagesFilter):
    """
    Returns only pinned messages
    
    """

    ID: str = Field("searchMessagesFilterPinned", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterPinned:
        return SearchMessagesFilterPinned.construct(**q)


class SearchMessagesFilterUnreadMention(SearchMessagesFilter):
    """
    Returns only messages with unread mentions of the current user, or messages that are replies to their messages. When using this filter the results can't be additionally filtered by a query, a message thread or by the sending user
    
    """

    ID: str = Field("searchMessagesFilterUnreadMention", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterUnreadMention:
        return SearchMessagesFilterUnreadMention.construct(**q)


class SearchMessagesFilterUrl(SearchMessagesFilter):
    """
    Returns only messages containing URLs
    
    """

    ID: str = Field("searchMessagesFilterUrl", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterUrl:
        return SearchMessagesFilterUrl.construct(**q)


class SearchMessagesFilterVideo(SearchMessagesFilter):
    """
    Returns only video messages
    
    """

    ID: str = Field("searchMessagesFilterVideo", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterVideo:
        return SearchMessagesFilterVideo.construct(**q)


class SearchMessagesFilterVideoNote(SearchMessagesFilter):
    """
    Returns only video note messages
    
    """

    ID: str = Field("searchMessagesFilterVideoNote", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterVideoNote:
        return SearchMessagesFilterVideoNote.construct(**q)


class SearchMessagesFilterVoiceAndVideoNote(SearchMessagesFilter):
    """
    Returns only voice and video note messages
    
    """

    ID: str = Field("searchMessagesFilterVoiceAndVideoNote", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterVoiceAndVideoNote:
        return SearchMessagesFilterVoiceAndVideoNote.construct(**q)


class SearchMessagesFilterVoiceNote(SearchMessagesFilter):
    """
    Returns only voice note messages
    
    """

    ID: str = Field("searchMessagesFilterVoiceNote", alias="@type")

    @staticmethod
    def read(q: dict) -> SearchMessagesFilterVoiceNote:
        return SearchMessagesFilterVoiceNote.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetMessageFileType(BaseObject):
    """
    Returns information about a file with messages exported from another app
    
    Params:
        message_file_head (:class:`str`)
            Beginning of the message file; up to 100 first lines
        
    """

    ID: str = Field("getMessageFileType", alias="@type")
    message_file_head: str

    @staticmethod
    def read(q: dict) -> GetMessageFileType:
        return GetMessageFileType.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ResetPasswordResult(BaseObject):
    """
    Represents result of 2-step verification password reset
    
    """

    ID: str = Field("resetPasswordResult", alias="@type")


class ResetPasswordResultDeclined(ResetPasswordResult):
    """
    The password reset request was declined
    
    Params:
        retry_date (:class:`int`)
            Point in time (Unix timestamp) when the password reset can be retried
        
    """

    ID: str = Field("resetPasswordResultDeclined", alias="@type")
    retry_date: int

    @staticmethod
    def read(q: dict) -> ResetPasswordResultDeclined:
        return ResetPasswordResultDeclined.construct(**q)


class ResetPasswordResultOk(ResetPasswordResult):
    """
    The password was reset
    
    """

    ID: str = Field("resetPasswordResultOk", alias="@type")

    @staticmethod
    def read(q: dict) -> ResetPasswordResultOk:
        return ResetPasswordResultOk.construct(**q)


class ResetPasswordResultPending(ResetPasswordResult):
    """
    The password reset request is pending
    
    Params:
        pending_reset_date (:class:`int`)
            Point in time (Unix timestamp) after which the password can be reset immediately using resetPassword
        
    """

    ID: str = Field("resetPasswordResultPending", alias="@type")
    pending_reset_date: int

    @staticmethod
    def read(q: dict) -> ResetPasswordResultPending:
        return ResetPasswordResultPending.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetBasicGroup(BaseObject):
    """
    Returns information about a basic group by its identifier. This is an offline request if the current user is not a bot
    
    Params:
        basic_group_id (:class:`int`)
            Basic group identifier
        
    """

    ID: str = Field("getBasicGroup", alias="@type")
    basic_group_id: int

    @staticmethod
    def read(q: dict) -> GetBasicGroup:
        return GetBasicGroup.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetGroupsInCommon(BaseObject):
    """
    Returns a list of common group chats with a given user. Chats are sorted by their type and creation date
    
    Params:
        user_id (:class:`int`)
            User identifier
        
        offset_chat_id (:class:`int`)
            Chat identifier starting from which to return chats; use 0 for the first request
        
        limit (:class:`int`)
            The maximum number of chats to be returned; up to 100
        
    """

    ID: str = Field("getGroupsInCommon", alias="@type")
    user_id: int
    offset_chat_id: int
    limit: int

    @staticmethod
    def read(q: dict) -> GetGroupsInCommon:
        return GetGroupsInCommon.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .chat_event_action import ChatEventAction
from ..base_object import BaseObject


class ChatEvent(BaseObject):
    """
    Represents a chat event
    
    Params:
        id (:class:`int`)
            Chat event identifier
        
        date (:class:`int`)
            Point in time (Unix timestamp) when the event happened
        
        user_id (:class:`int`)
            Identifier of the user who performed the action that triggered the event
        
        action (:class:`ChatEventAction`)
            Action performed by the user
        
    """

    ID: str = Field("chatEvent", alias="@type")
    id: int
    date: int
    user_id: int
    action: ChatEventAction

    @staticmethod
    def read(q: dict) -> ChatEvent:
        return ChatEvent.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class CallbackQueryAnswer(BaseObject):
    """
    Contains a bot's answer to a callback query
    
    Params:
        text (:class:`str`)
            Text of the answer
        
        show_alert (:class:`bool`)
            True, if an alert should be shown to the user instead of a toast notification
        
        url (:class:`str`)
            URL to be opened
        
    """

    ID: str = Field("callbackQueryAnswer", alias="@type")
    text: str
    show_alert: bool
    url: str

    @staticmethod
    def read(q: dict) -> CallbackQueryAnswer:
        return CallbackQueryAnswer.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .chat_photo_info import ChatPhotoInfo
from .chat_type import ChatType
from ..base_object import BaseObject


class ChatInviteLinkInfo(BaseObject):
    """
    Contains information about a chat invite link
    
    Params:
        chat_id (:class:`int`)
            Chat identifier of the invite link; 0 if the user has no access to the chat before joining
        
        accessible_for (:class:`int`)
            If non-zero, the amount of time for which read access to the chat will remain available, in seconds
        
        type_ (:class:`ChatType`)
            Contains information about the type of the chat
        
        title (:class:`str`)
            Title of the chat
        
        photo (:class:`ChatPhotoInfo`)
            Chat photo; may be null
        
        member_count (:class:`int`)
            Number of members in the chat
        
        member_user_ids (:obj:`list[int]`)
            User identifiers of some chat members that may be known to the current user
        
        is_public (:class:`bool`)
            True, if the chat is a public supergroup or channel, i.e. it has a username or it is a location-based supergroup
        
    """

    ID: str = Field("chatInviteLinkInfo", alias="@type")
    chat_id: int
    accessible_for: int
    type_: ChatType = Field(..., alias='type')
    title: str
    photo: typing.Optional[ChatPhotoInfo] = None
    member_count: int
    member_user_ids: list[int]
    is_public: bool

    @staticmethod
    def read(q: dict) -> ChatInviteLinkInfo:
        return ChatInviteLinkInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetGroupCallStreamSegment(BaseObject):
    """
    Returns a file with a segment of a group call stream in a modified OGG format
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        time_offset (:class:`int`)
            Point in time when the stream segment begins; Unix timestamp in milliseconds
        
        scale (:class:`int`)
            Segment duration scale; 0-1. Segment's duration is 1000/(2**scale) milliseconds
        
    """

    ID: str = Field("getGroupCallStreamSegment", alias="@type")
    group_call_id: int
    time_offset: int
    scale: int

    @staticmethod
    def read(q: dict) -> GetGroupCallStreamSegment:
        return GetGroupCallStreamSegment.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import OrderInfo


class ValidateOrderInfo(BaseObject):
    """
    Validates the order information provided by a user and returns the available shipping options for a flexible invoice
    
    Params:
        chat_id (:class:`int`)
            Chat identifier of the Invoice message
        
        message_id (:class:`int`)
            Message identifier
        
        order_info (:class:`OrderInfo`)
            The order information, provided by the user
        
        allow_save (:class:`bool`)
            True, if the order information can be saved
        
    """

    ID: str = Field("validateOrderInfo", alias="@type")
    chat_id: int
    message_id: int
    order_info: OrderInfo
    allow_save: bool

    @staticmethod
    def read(q: dict) -> ValidateOrderInfo:
        return ValidateOrderInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SetPassword(BaseObject):
    """
    Changes the password for the user. If a new recovery email address is specified, then the change will not be applied until the new recovery email address is confirmed
    
    Params:
        old_password (:class:`str`)
            Previous password of the user
        
        new_password (:class:`str`)
            New password of the user; may be empty to remove the password
        
        new_hint (:class:`str`)
            New password hint; may be empty
        
        set_recovery_email_address (:class:`bool`)
            Pass true if the recovery email address should be changed
        
        new_recovery_email_address (:class:`str`)
            New recovery email address; may be empty
        
    """

    ID: str = Field("setPassword", alias="@type")
    old_password: str
    new_password: str
    new_hint: str
    set_recovery_email_address: bool
    new_recovery_email_address: str

    @staticmethod
    def read(q: dict) -> SetPassword:
        return SetPassword.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .chat_invite_link_info import ChatInviteLinkInfo
from ..base_object import BaseObject


class TMeUrlType(BaseObject):
    """
    Describes the type of a URL linking to an internal Telegram entity
    
    """

    ID: str = Field("tMeUrlType", alias="@type")


class TMeUrlTypeChatInvite(TMeUrlType):
    """
    A chat invite link
    
    Params:
        info (:class:`ChatInviteLinkInfo`)
            Chat invite link info
        
    """

    ID: str = Field("tMeUrlTypeChatInvite", alias="@type")
    info: ChatInviteLinkInfo

    @staticmethod
    def read(q: dict) -> TMeUrlTypeChatInvite:
        return TMeUrlTypeChatInvite.construct(**q)


class TMeUrlTypeStickerSet(TMeUrlType):
    """
    A URL linking to a sticker set
    
    Params:
        sticker_set_id (:class:`int`)
            Identifier of the sticker set
        
    """

    ID: str = Field("tMeUrlTypeStickerSet", alias="@type")
    sticker_set_id: int

    @staticmethod
    def read(q: dict) -> TMeUrlTypeStickerSet:
        return TMeUrlTypeStickerSet.construct(**q)


class TMeUrlTypeSupergroup(TMeUrlType):
    """
    A URL linking to a public supergroup or channel
    
    Params:
        supergroup_id (:class:`int`)
            Identifier of the supergroup or channel
        
    """

    ID: str = Field("tMeUrlTypeSupergroup", alias="@type")
    supergroup_id: int

    @staticmethod
    def read(q: dict) -> TMeUrlTypeSupergroup:
        return TMeUrlTypeSupergroup.construct(**q)


class TMeUrlTypeUser(TMeUrlType):
    """
    A URL linking to a user
    
    Params:
        user_id (:class:`int`)
            Identifier of the user
        
    """

    ID: str = Field("tMeUrlTypeUser", alias="@type")
    user_id: int

    @staticmethod
    def read(q: dict) -> TMeUrlTypeUser:
        return TMeUrlTypeUser.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import InputFile


class RemoveStickerFromSet(BaseObject):
    """
    Removes a sticker from the set to which it belongs; for bots only. The sticker set must have been created by the bot
    
    Params:
        sticker (:class:`InputFile`)
            Sticker
        
    """

    ID: str = Field("removeStickerFromSet", alias="@type")
    sticker: InputFile

    @staticmethod
    def read(q: dict) -> RemoveStickerFromSet:
        return RemoveStickerFromSet.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import AccountTtl


class SetAccountTtl(BaseObject):
    """
    Changes the period of inactivity after which the account of the current user will automatically be deleted
    
    Params:
        ttl (:class:`AccountTtl`)
            New account TTL
        
    """

    ID: str = Field("setAccountTtl", alias="@type")
    ttl: AccountTtl

    @staticmethod
    def read(q: dict) -> SetAccountTtl:
        return SetAccountTtl.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from ..base_object import BaseObject
from ..types import ChatReportReason


class ReportChat(BaseObject):
    """
    Reports a chat to the Telegram moderators. A chat can be reported only from the chat action bar, or if this is a private chat with a bot, a private chat with a user sharing their location, a supergroup, or a channel, since other chats can't be checked by moderators
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        message_ids (:obj:`list[int]`)
            Identifiers of reported messages, if any
        
        reason (:class:`ChatReportReason`)
            The reason for reporting the chat
        
        text (:class:`str`)
            Additional report details; 0-1024 characters
        
    """

    ID: str = Field("reportChat", alias="@type")
    chat_id: int
    message_ids: list[int]
    reason: ChatReportReason
    text: typing.Optional[str] = Field(None, max_length=1024)

    @staticmethod
    def read(q: dict) -> ReportChat:
        return ReportChat.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class Session(BaseObject):
    """
    Contains information about one session in a Telegram application used by the current user. Sessions should be shown to the user in the returned order
    
    Params:
        id (:class:`int`)
            Session identifier
        
        is_current (:class:`bool`)
            True, if this session is the current session
        
        is_password_pending (:class:`bool`)
            True, if a password is needed to complete authorization of the session
        
        api_id (:class:`int`)
            Telegram API identifier, as provided by the application
        
        application_name (:class:`str`)
            Name of the application, as provided by the application
        
        application_version (:class:`str`)
            The version of the application, as provided by the application
        
        is_official_application (:class:`bool`)
            True, if the application is an official application or uses the api_id of an official application
        
        device_model (:class:`str`)
            Model of the device the application has been run or is running on, as provided by the application
        
        platform (:class:`str`)
            Operating system the application has been run or is running on, as provided by the application
        
        system_version (:class:`str`)
            Version of the operating system the application has been run or is running on, as provided by the application
        
        log_in_date (:class:`int`)
            Point in time (Unix timestamp) when the user has logged in
        
        last_active_date (:class:`int`)
            Point in time (Unix timestamp) when the session was last used
        
        ip (:class:`str`)
            IP address from which the session was created, in human-readable format
        
        country (:class:`str`)
            A two-letter country code for the country from which the session was created, based on the IP address
        
        region (:class:`str`)
            Region code from which the session was created, based on the IP address
        
    """

    ID: str = Field("session", alias="@type")
    id: int
    is_current: bool
    is_password_pending: bool
    api_id: int
    application_name: str
    application_version: str
    is_official_application: bool
    device_model: str
    platform: str
    system_version: str
    log_in_date: int
    last_active_date: int
    ip: str
    country: str
    region: str

    @staticmethod
    def read(q: dict) -> Session:
        return Session.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class Seconds(BaseObject):
    """
    Contains a value representing a number of seconds
    
    Params:
        seconds (:class:`float`)
            Number of seconds
        
    """

    ID: str = Field("seconds", alias="@type")
    seconds: float

    @staticmethod
    def read(q: dict) -> Seconds:
        return Seconds.construct(**q)



import logging
import platform
import sys
import typing
from ctypes import *
from ctypes.util import find_library
from enum import IntEnum

import pkg_resources
import ujson

TDLIB_VERSION = '1.7.6'
TDLIB_MAX_INT = 2 ** 63 - 1
log_message_callback_type = CFUNCTYPE(None, c_int, c_char_p)


def _get_tdjson_lib_path() -> str:
    tdjson_path = find_library('tdjson')

    if tdjson_path is not None:
        return tdjson_path

    if platform.system().lower() == 'darwin':
        lib_name = f"libtdjson.{TDLIB_VERSION}.dylib"
    elif platform.system().lower() == 'windows':
        lib_name = f"libtdjson.{TDLIB_VERSION}.dll"
    else:
        # By default tdlib is built with this name
        lib_name = f"libtdjson.so.{TDLIB_VERSION}"

    return pkg_resources.resource_filename('aiotdlib', f'tdlib/{lib_name}')


class TDLibLogVerbosity(IntEnum):
    FATAL = 0
    ERROR = 1
    WARNING = 2
    INFO = 3
    DEBUG = 4
    VERBOSE = 5
    MAXIMUM = 1023


class TDJson:
    logger: logging.Logger

    # TDLib functions typings
    __tdjson: CDLL
    __td_create_client_id: typing.Callable[[], int]
    __td_receive: typing.Callable[[float], bytes]
    __td_send: typing.Callable[[int, bytes], None]
    __td_execute: typing.Callable[[bytes], bytes]
    __td_set_log_verbosity_level: typing.Callable[[int], None]
    __td_set_log_message_callback: typing.Callable[[log_message_callback_type], None]

    td_json_client: typing.Optional[int] = None

    def __init__(self, library_path: str = None, verbosity: TDLibLogVerbosity = TDLibLogVerbosity.ERROR) -> None:
        if library_path is None:
            library_path = _get_tdjson_lib_path()

        if library_path is None:
            raise ValueError('Unable to find TD library binaries')

        self.logger = logging.getLogger(__name__)
        self.logger.info('Using shared library "%s"', library_path)
        self.initialize(library_path, verbosity)

    def __del__(self) -> None:
        try:
            self.stop()
        except:
            pass

    def inject_library(self, library_path: str):
        # load TDLib functions from shared library
        self.__tdjson = CDLL(library_path)

        # TDJSON_EXPORT int td_create_client_id();
        self.__td_create_client_id = self.__tdjson.td_create_client_id
        self.__td_create_client_id.restype = c_int
        self.__td_create_client_id.argtypes = []

        # TDJSON_EXPORT const char *td_receive(double timeout);
        self.__td_receive = self.__tdjson.td_receive
        self.__td_receive.restype = c_char_p
        self.__td_receive.argtypes = [c_double]

        # TDJSON_EXPORT void td_send(int client_id, const char *request);
        self.__td_send = self.__tdjson.td_send
        self.__td_send.restype = None
        self.__td_send.argtypes = [c_int, c_char_p]

        # TDJSON_EXPORT const char *td_execute(const char *request);
        self.__td_execute = self.__tdjson.td_execute
        self.__td_execute.restype = c_char_p
        self.__td_execute.argtypes = [c_char_p]

        # td_set_log_message_callback
        self.__td_set_log_message_callback = self.__tdjson.td_set_log_message_callback
        self.__td_set_log_message_callback.restype = None
        self.__td_set_log_message_callback.argtypes = [log_message_callback_type]

        # td_set_log_verbosity_level
        self.__td_set_log_verbosity_level = self.__tdjson.td_set_log_verbosity_level
        self.__td_set_log_verbosity_level.restype = None
        self.__td_set_log_verbosity_level.argtypes = [c_int]

    def initialize(self, library_path: str, verbosity: TDLibLogVerbosity) -> None:
        self.inject_library(library_path)

        if bool(self.td_json_client):
            raise RuntimeError('TDJson instance is already initialized')

        self.__td_set_log_verbosity_level(int(verbosity))
        self.__td_set_log_message_callback(log_message_callback_type(self.__log_message_callback))
        self.td_json_client = self.__td_create_client_id()

    def send(self, query: dict):
        if not bool(self.td_json_client):
            raise RuntimeError('Instance is not initialized')

        dumped_query = ujson.dumps(query, ensure_ascii=False).encode('utf-8')
        self.logger.debug(f'[me >>>] Sending {dumped_query}')
        self.__td_send(self.td_json_client, dumped_query)

    def execute(self, query: dict) -> typing.Optional[dict]:
        if not bool(self.td_json_client):
            raise RuntimeError('Instance is not initialized')

        dumped_query = ujson.dumps(query, ensure_ascii=False).encode('utf-8')
        self.logger.debug(f'Executing query {dumped_query} as client {self.td_json_client}')
        result = self.__td_execute(dumped_query)

        if result:
            result = ujson.loads(result)

        return result

    def receive(self, timeout: float = 10.0) -> typing.Optional[dict]:
        if not bool(self.td_json_client):
            raise RuntimeError('Instance is not initialized')

        result = self.__td_receive(timeout)

        if bool(result):
            self.logger.debug(f'[me <<<] Received {result}')
            result = ujson.loads(result)

            # Each returned object will have an "@client_id" field,
            # containing the identifier of the client for which a response or an update was received.
            if result.pop('@client_id', None) != self.td_json_client:
                return None

        return result

    def stop(self) -> None:
        # TDLib client instances are destroyed automatically after they are closed, so we need to send close
        self.send({'@type': 'close'})

    def __log_message_callback(self, verbosity_level: int, message: str) -> None:
        if verbosity_level == TDLibLogVerbosity.FATAL:
            self.logger.error('TDLib fatal error: %s', message)

        sys.stdout.flush()



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetGameHighScores(BaseObject):
    """
    Returns the high scores for a game and some part of the high score table in the range of the specified user; for bots only
    
    Params:
        chat_id (:class:`int`)
            The chat that contains the message with the game
        
        message_id (:class:`int`)
            Identifier of the message
        
        user_id (:class:`int`)
            User identifier
        
    """

    ID: str = Field("getGameHighScores", alias="@type")
    chat_id: int
    message_id: int
    user_id: int

    @staticmethod
    def read(q: dict) -> GetGameHighScores:
        return GetGameHighScores.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class StatisticalGraph(BaseObject):
    """
    Describes a statistical graph
    
    """

    ID: str = Field("statisticalGraph", alias="@type")


class StatisticalGraphAsync(StatisticalGraph):
    """
    The graph data to be asynchronously loaded through getStatisticalGraph
    
    Params:
        token (:class:`str`)
            The token to use for data loading
        
    """

    ID: str = Field("statisticalGraphAsync", alias="@type")
    token: str

    @staticmethod
    def read(q: dict) -> StatisticalGraphAsync:
        return StatisticalGraphAsync.construct(**q)


class StatisticalGraphData(StatisticalGraph):
    """
    A graph data
    
    Params:
        json_data (:class:`str`)
            Graph data in JSON format
        
        zoom_token (:class:`str`)
            If non-empty, a token which can be used to receive a zoomed in graph
        
    """

    ID: str = Field("statisticalGraphData", alias="@type")
    json_data: str
    zoom_token: str

    @staticmethod
    def read(q: dict) -> StatisticalGraphData:
        return StatisticalGraphData.construct(**q)


class StatisticalGraphError(StatisticalGraph):
    """
    An error message to be shown to the user instead of the graph
    
    Params:
        error_message (:class:`str`)
            The error message
        
    """

    ID: str = Field("statisticalGraphError", alias="@type")
    error_message: str

    @staticmethod
    def read(q: dict) -> StatisticalGraphError:
        return StatisticalGraphError.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .message_reply_info import MessageReplyInfo
from ..base_object import BaseObject


class MessageInteractionInfo(BaseObject):
    """
    Contains information about interactions with a message
    
    Params:
        view_count (:class:`int`)
            Number of times the message was viewed
        
        forward_count (:class:`int`)
            Number of times the message was forwarded
        
        reply_info (:class:`MessageReplyInfo`)
            Contains information about direct or indirect replies to the message; may be null. Currently, available only in channels with a discussion supergroup and discussion supergroups for messages, which are not replies itself
        
    """

    ID: str = Field("messageInteractionInfo", alias="@type")
    view_count: int
    forward_count: int
    reply_info: typing.Optional[MessageReplyInfo] = None

    @staticmethod
    def read(q: dict) -> MessageInteractionInfo:
        return MessageInteractionInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import NetworkStatisticsEntry


class AddNetworkStatistics(BaseObject):
    """
    Adds the specified data to data usage statistics. Can be called before authorization
    
    Params:
        entry (:class:`NetworkStatisticsEntry`)
            The network statistics entry with the data to be added to statistics
        
    """

    ID: str = Field("addNetworkStatistics", alias="@type")
    entry: NetworkStatisticsEntry

    @staticmethod
    def read(q: dict) -> AddNetworkStatistics:
        return AddNetworkStatistics.construct(**q)



from __future__ import annotations

import asyncio
import enum
import hashlib
import logging
import os
import signal
import sys
import typing
import uuid
from collections import AsyncIterator
from functools import (
    partial,
    update_wrapper,
)
from pathlib import Path
from typing import (
    Optional,
    TypeVar,
    Union,
)

import pydantic.errors
from pydantic import (
    BaseModel,
    validator,
)

from . import __version__
from .api import (
    API,
    AioTDLibError,
    AuthorizationStateClosed,
    BaseObject,
    BasicGroup,
    BasicGroupFullInfo,
    Chat,
    ChatTypeBasicGroup,
    ChatTypePrivate,
    ChatTypeSecret,
    ChatTypeSupergroup,
    Error,
    FormattedText,
    InputFileLocal,
    InputFileRemote,
    InputMessageAnimation,
    InputMessageAudio,
    InputMessageContent,
    InputMessageDocument,
    InputMessagePhoto,
    InputMessageText,
    InputMessageVideo,
    InputMessageVideoNote,
    InputMessageVoiceNote,
    InputThumbnail,
    Message,
    MessageSchedulingStateSendAtDate,
    MessageSchedulingStateSendWhenOnline,
    MessageSendOptions,
    MessageSendingStatePending,
    Messages,
    ParseTextEntities,
    PhoneNumberAuthenticationSettings,
    ProxyType,
    ProxyTypeHttp,
    ProxyTypeMtproto,
    ProxyTypeSocks5,
    ReplyMarkup,
    SecretChat,
    Supergroup,
    SupergroupFullInfo,
    TdlibParameters,
    TextParseModeHTML,
    TextParseModeMarkdown,
    UpdateAuthorizationState,
    UpdateMessageSendSucceeded,
    User,
    UserFullInfo,
)
from .client_cache import ClientCache
from .filters import Filters
from .handlers import (
    FilterCallable,
    Handler,
    HandlerCallable,
)
from .middlewares import MiddlewareCallable
from .tdjson import (
    TDJson,
    TDLibLogVerbosity,
)
from .utils import (
    PendingRequest,
    ainput,
    str_to_base64,
)

RequestResult = TypeVar('RequestResult', bound=BaseObject)
ExecuteResult = TypeVar('ExecuteResult', bound=BaseObject)

ChatInfo = Union[
    User,
    UserFullInfo,
    BasicGroup,
    BasicGroupFullInfo,
    Supergroup,
    SupergroupFullInfo,
    SecretChat
]


class ClientProxyType(str, enum.Enum):
    MTPROTO = 'mtproto'
    HTTP = 'http'
    SOCKS5 = 'socks5'


class ClientProxySettings(BaseModel):
    """
    Universal proxy settings object for all proxy types

    Params:
        host (:class:`str`)
            Proxy server IP address

        port (:class:`int`)
            Proxy server port

        type (:class:`ClientProxySettingsType`)
            Proxy type

        username (:class:`str`)
            Username for logging in; may be empty

        password (:class:`str`)
            Password for logging in; may be empty

        http_only (:class:`bool`)
            Pass true if the proxy supports only HTTP requests
            and doesn't support transparent TCP connections via HTTP CONNECT method

        secret (:class:`str`)
            The proxy's secret in hexadecimal encoding

    """

    host: str
    port: int
    type: ClientProxyType = ClientProxyType.SOCKS5
    username: Optional[str] = None
    password: Optional[str] = None
    http_only: bool = False
    secret: Optional[str] = None

    @validator('secret', pre=True, always=True)
    def validate_secret(cls, secret: str, values):
        if values.get('type') == ClientProxyType.MTPROTO and secret is None:
            raise pydantic.errors.MissingError

        return secret


class Client:
    logger: logging.Logger = None
    loop: asyncio.AbstractEventLoop = None
    __stop_idle_event: asyncio.Event = None

    def __init__(
            self,
            api_id: int,
            api_hash: str,
            database_encryption_key: Union[str, bytes] = 'aiotdlib',
            phone_number: str = None,
            bot_token: str = None,
            use_test_dc: bool = False,
            system_language_code: str = 'en',
            device_model: str = 'aiotdlib',
            system_version: str = "",
            application_version: str = __version__,
            files_directory: str = None,
            first_name: str = None,
            last_name: str = None,
            library_path: str = None,
            tdlib_verbosity: TDLibLogVerbosity = TDLibLogVerbosity.ERROR,
            debug: bool = False,
            parse_mode: str = 'html',
            proxy_settings: ClientProxySettings = None
    ):
        """
            Params:
                api_id (:class:`int`)
                    Application identifier for Telegram API access, which can be obtained at https://my.telegram.org

                api_hash (:class:`str`)
                    Application identifier hash for Telegram API access,
                    which can be obtained at https://my.telegram.org

                database_encryption_key (:class:`str`)
                    Encryption key of local session database. Default: aiotdlib

                phone_number (:class:`str`)
                    The phone number of the user, in international format.
                    Either phone_number or bot_token MUST be passed. ValueError would be raised otherwise

                bot_token (:class:`str`)
                    The bot token.
                    Either phone_number or bot_token MUST be passed. ValueError would be raised otherwise

                use_test_dc (:class:`bool`)
                    If set to true, the Telegram test environment will be used instead of the production environment

                system_language_code (:class:`str`)
                    IETF language tag of the user's operating system language; must be non-empty

                device_model (:class:`str`)
                    Model of the device the application is being run on; must be non-empty

                system_version (:class:`str`)
                    Version of the operating system the application is being run on.
                    If empty, the version is automatically detected by TDLib

                application_version (:class:`str`)
                    Application version; must be non-empty

                files_directory (:class:`str`)
                    The path to the directory for storing files. Default: .aiotdlib/

                first_name (:class:`str`)
                    First name of new account if account with passed phone_number does not exist

                last_name (:class:`str`)
                    Last name of new account if account with passed phone_number does not exist

                library_path (:class:`str`)
                    Path to TDLib binary. By default binary included in package is used

                tdlib_verbosity (:class:`str`)
                    Verbosity level of TDLib itself.
                    Default: 2 (WARNING) for more info look at (:class:`TDLibLogVerbosity`)

                debug (:class:`bool`)
                    When set to true all request and responses would be logged in console with DEBUG level

                parse_mode (:class:`str`)
                    Default parse mode for high-level methods like send_message. Default: html

                proxy_settings (:class:`ClientProxySettings`)
                    Settings for proxying telegram connection

        """
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.DEBUG if debug else logging.INFO)

        if not bool(bot_token) and not bool(phone_number):
            raise ValueError('Either phone_number or bot_token should be specified')

        self.api_id = api_id
        self.api_hash = api_hash
        self.database_encryption_key = database_encryption_key
        self.phone_number = phone_number
        self.bot_token = bot_token
        self.use_test_dc = use_test_dc
        self.device_model = device_model
        self.application_version = application_version
        self.system_version = system_version
        self.system_language_code = system_language_code
        self.first_name = first_name
        self.last_name = last_name
        self.parse_mode = 'markdown' if (bool(parse_mode) and parse_mode.lower() == 'markdown') else 'html'
        self.proxy_settings = proxy_settings

        md5_hash = hashlib.md5()
        md5_hash.update((self.phone_number or self.bot_token).encode('utf-8'))
        directory_name = md5_hash.hexdigest()

        self.debug = debug

        if bool(files_directory):
            self.files_directory = str(os.path.join(files_directory, '.aiotdlib', directory_name))
        else:
            self.files_directory = str(os.path.join(Path(sys.argv[0]).parent, '.aiotdlib', directory_name))

        self.__tdjson = TDJson(library_path=library_path, verbosity=tdlib_verbosity)
        self.__current_authorization_state = None
        self.__is_authorized = False
        self.__running = False
        self.__pending_requests: dict[str, PendingRequest] = {}
        # "{chat_id}_{message_id}" will be used as key
        self.__pending_messages: dict[str, Message] = {}

        # For handlers registration
        self.__updates_handlers: dict[str, set[Handler]] = {}
        self.__middlewares: list[MiddlewareCallable] = []
        self.__middlewares_handlers: list[MiddlewareCallable] = []

        self.api = API(self)
        self.cache = ClientCache(self)
        self.first_time_auth = False

    # Magic methods
    async def __aenter__(self) -> 'Client':
        return await self.start()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()

    async def __call_handler(self, handler: Handler, update: BaseObject):
        try:
            await handler(self, update)
        except Exception as e:
            self.logger.error(e, exc_info=True)

    async def __call_handlers(self, update: BaseObject):
        tasks = []
        tasks.extend(self.__call_handler(h, update) for h in self.__updates_handlers.get(update.ID, []))
        tasks.extend(self.__call_handler(h, update) for h in self.__updates_handlers.get('*', []))
        # Running all handlers concurrently and independently
        await asyncio.gather(*tasks, return_exceptions=True)

    async def __handle_update(self, update: BaseObject):
        if len(self.__middlewares_handlers) == 0:
            return await self.__call_handlers(update)

        async def __fn(*_, **__):
            return await self.__call_handlers(update)

        call_next = __fn

        for m in self.__middlewares_handlers:
            call_next = update_wrapper(partial(m, call_next=call_next), call_next)

        return await call_next(self, update)

    async def __handle_pending_request(self, update: BaseObject):
        request_id = update.EXTRA.get('request_id')

        if not bool(request_id) and update.ID in ['updateAuthorizationState']:
            request_id = update.ID

        if bool(request_id):
            pending_request = self.__pending_requests.get(request_id)

            if bool(pending_request):
                if isinstance(update, Message) and isinstance(update.sending_state, MessageSendingStatePending):
                    self.__pending_messages[f"{update.chat_id}_{update.id}"] = update
                else:
                    self.__pending_requests.pop(request_id)
                    pending_request.set_update(update)

        if isinstance(update, UpdateMessageSendSucceeded):
            pending_message = self.__pending_messages.pop(
                f"{update.message.chat_id}_{update.old_message_id}",
                None
            )

            if bool(pending_message):
                request_id = pending_message.EXTRA.get('request_id')
                pending_request = self.__pending_requests.get(request_id)

                if bool(pending_request):
                    update.message.EXTRA['request_id'] = request_id
                    self.__pending_requests.pop(request_id)
                    pending_request.set_update(update.message)

    async def __updates_loop(self):
        try:
            while self.__running:
                update = await self.receive()

                if not bool(update):
                    continue

                if isinstance(update, UpdateAuthorizationState):
                    if isinstance(update.authorization_state, AuthorizationStateClosed):
                        self.logger.warning('Session was terminated!')
                        self.loop.create_task(self.stop())
                        return

                self.loop.create_task(self.__handle_pending_request(update))
                self.loop.create_task(self.__handle_update(update))
        except (asyncio.CancelledError, KeyboardInterrupt):
            raise
        except Exception as e:
            self.logger.error(f'Unhandled exception occurred!. {e.__class__.__qualname__}. {e}', exc_info=True)

    async def __close(self):
        if not bool(self.__running):
            return

        if bool(self.__tdjson):
            self.logger.info('Gracefully closing TDLib connection')
            self.__tdjson.stop()

    async def __auth_start(self) -> RequestResult:
        return await self.api.get_authorization_state(request_id="updateAuthorizationState")

    async def __set_tdlib_parameters(self) -> RequestResult:
        return await self.api.set_tdlib_parameters(
            parameters=TdlibParameters(
                use_test_dc=self.use_test_dc,
                database_directory=str(f"{self.files_directory}/database"),
                files_directory=str(f"{self.files_directory}/files"),
                use_file_database=True,
                use_chat_info_database=True,
                use_message_database=True,
                use_secret_chats=True,
                api_id=self.api_id,
                api_hash=self.api_hash,
                system_language_code=self.system_language_code,
                device_model=self.device_model,
                system_version=self.system_version,
                application_version=self.application_version,
                enable_storage_optimizer=True,
                ignore_file_names=True
            ),
            request_id="updateAuthorizationState"
        )

    async def __check_database_encryption_key(self) -> RequestResult:
        self.logger.info('Sending encryption key')
        result = await self.api.check_database_encryption_key(
            encryption_key=str_to_base64(self.database_encryption_key),
            request_id="updateAuthorizationState"
        )

        await self.__setup_proxy()

        return result

    async def __set_authentication_phone_number_or_check_bot_token(self) -> RequestResult:
        self.first_time_auth = True

        if self.phone_number:
            return await self.__set_authentication_phone_number()
        elif self.bot_token:
            return await self.__check_authentication_bot_token()
        else:
            raise RuntimeError('Unknown mode: both bot_token and phone_number are None')

    async def __set_authentication_phone_number(self) -> RequestResult:
        self.logger.info('Sending phone number')
        return await self.api.set_authentication_phone_number(
            phone_number=self.phone_number,
            settings=PhoneNumberAuthenticationSettings(
                is_current_phone_number=True,
                allow_flash_call=False,
                allow_sms_retriever_api=False
            ),
            request_id="updateAuthorizationState"
        )

    async def __check_authentication_bot_token(self) -> RequestResult:
        self.logger.info('Sending bot token')
        return await self.api.check_authentication_bot_token(
            self.bot_token,
            request_id="updateAuthorizationState"
        )

    async def __auth_get_code(self) -> str:
        code = ""

        while len(code) != 5 or not code.isdigit():
            code = await ainput('Enter code:')

        return code

    async def __auth_get_password(self) -> str:
        password = ""

        if not bool(password):
            password = await ainput('Enter 2FA password:', secured=True)

        return password

    async def __auth_get_first_name(self) -> str:
        first_name = self.first_name or ""

        while not bool(first_name) or len(first_name) > 64:
            first_name = await ainput('Enter first name:')

        return first_name

    async def __auth_get_last_name(self) -> str:
        last_name = self.last_name or ""

        if not bool(last_name):
            last_name = await ainput('Enter last name:')

        return last_name

    async def __check_authentication_code(self) -> RequestResult:
        self.first_time_auth = True

        code = await self.__auth_get_code()
        self.logger.info(f'Sending code {code}')

        return await self.api.check_authentication_code(
            code=code,
            request_id="updateAuthorizationState"
        )

    async def __register_user(self) -> RequestResult:
        self.first_time_auth = True

        first_name = await self.__auth_get_first_name()
        last_name = await self.__auth_get_last_name()
        self.logger.info(f'Registering new user in telegram as {first_name} {last_name or ""}'.strip())

        return await self.api.register_user(
            first_name=first_name,
            last_name=last_name,
            request_id="updateAuthorizationState"
        )

    async def __check_authentication_password(self) -> RequestResult:
        self.first_time_auth = True

        password = await self.__auth_get_password()
        self.logger.info('Sending password')

        return await self.api.check_authentication_password(
            password=password,
            request_id="updateAuthorizationState"
        )

    async def __auth_completed(self):
        self.__pending_requests.pop('updateAuthorizationState', None)

        if not bool(self.bot_token) and self.first_time_auth:
            # Update chats list in cache after successful authorization
            await self.get_main_list_chats(limit=500)

        self.__is_authorized = True
        self.logger.info('Authorization is completed')

    async def __auth_logging_out(self):
        self.logger.info('Auth session is logging out')

    async def __auth_closing(self):
        self.logger.info('Auth session is closing')

    async def __auth_closed(self):
        self.logger.info('Auth session is closed')

    async def __stop_signal_handler(self, signum: int) -> None:
        self.logger.info('Signal %s received!', signum)
        await self.stop()

    async def __setup_proxy(self):
        if not bool(self.proxy_settings):
            # If proxy is not set disabling all configured proxy
            await self.api.disable_proxy()
            return

        self.logger.info('Retrieving all proxies list')
        result = await self.api.get_proxies()

        proxy_type_by_class: dict[typing.Type[ProxyType], str] = {
            ProxyTypeSocks5: 'socks5',
            ProxyTypeMtproto: 'mtproto',
            ProxyTypeHttp: 'http',
        }

        if self.debug:
            proxies_list_string = "\n".join(
                f"{'* ' if p.is_enabled else ''}[{proxy_type_by_class.get(p.type_.__class__)}] {p.server}:{p.port}"
                for p in result.proxies
            )
            self.logger.info(
                f'{len(result.proxies)} proxies already set up:\n'
                f'{proxies_list_string}'
                f'\n'
            )

        for p in result.proxies:
            if (
                    p.server == self.proxy_settings.host and
                    p.port == self.proxy_settings.port and
                    proxy_type_by_class.get(p.type_.__class__) == self.proxy_settings.type
            ):
                if not p.is_enabled:
                    await self.api.enable_proxy(p.id)

                return

        if self.proxy_settings.type == ClientProxyType.HTTP:
            proxy_type = ProxyTypeHttp.construct(
                username=self.proxy_settings.username,
                password=self.proxy_settings.password,
                http_only=self.proxy_settings.http_only
            )
        elif self.proxy_settings.type == ClientProxyType.MTPROTO:
            proxy_type = ProxyTypeMtproto.construct(secret=self.proxy_settings.secret)
        elif self.proxy_settings.type == ClientProxyType.SOCKS5:
            proxy_type = ProxyTypeSocks5.construct(
                username=self.proxy_settings.username,
                password=self.proxy_settings.password,
            )
        else:
            raise ValueError(f'Unknown proxy type {self.proxy_settings.type}')

        self.logger.info(
            f"Configuring PROXY of type {self.proxy_settings.type.value}. "
            f"Server: {self.proxy_settings.host}:{self.proxy_settings.port}"
        )

        await self.api.add_proxy(
            enable=True,
            server=self.proxy_settings.host,
            port=self.proxy_settings.port,
            type_=proxy_type,
        )

    def add_event_handler(
            self,
            handler: HandlerCallable,
            update_type: str = API.Types.ANY,
            *,
            filters: FilterCallable = None
    ):
        """
            Registering event handler
            You can register many handlers for certain event type
        """
        if self.__updates_handlers.get(update_type) is None:
            self.__updates_handlers[update_type] = set()

        if handler not in self.__updates_handlers[update_type]:
            self.__updates_handlers[update_type].add(
                handler
                if isinstance(handler, Handler)
                else Handler(handler, filters=filters)
            )

    def on_event(self, update_type: str = API.Types.ANY, *, filters: FilterCallable = None):
        def decorator(function: HandlerCallable) -> HandlerCallable:
            self.add_event_handler(function, update_type, filters=filters)
            return function

        return decorator

    def remove_event_handler(self, handler: Handler, update_type: str = API.Types.ANY):
        if self.__updates_handlers.get(update_type) is None:
            return

        self.__updates_handlers.get(update_type).remove(handler)

    def add_middleware(self, middleware: MiddlewareCallable):
        """
            Register middleware.
            Note that middleware would be called for EVERY EVENT.
            Do not use them for long-running tasks as it could be heavy performance hit
            You can add as much middlewares as you want.
            They would be called in order you've added them
        """

        self.__middlewares.append(middleware)
        return middleware

    def text_message_handler(self, function: HandlerCallable = None):
        """
        Registers event handler with predefined filter Filters.text
        which allows only UpdateNewMessage with MessageText content

        Note: this method is universal and can be used directly or as decorator
        """
        if callable(function):
            self.add_event_handler(function, API.Types.UPDATE_NEW_MESSAGE, filters=Filters.text)
        else:
            return self.on_event(API.Types.UPDATE_NEW_MESSAGE, filters=Filters.text)

    def bot_command_handler(self, function: HandlerCallable = None, *, command: str = None):
        """
        Registers event handler with predefined filter Filters.bot_command
        which allows only UpdateNewMessage with MessageText content and text of message starts with "/"

        Note: this method is universal and can be used directly or as decorator
        """
        if callable(function):
            self.add_event_handler(function, API.Types.UPDATE_NEW_MESSAGE, filters=Filters.bot_command(command))
        else:
            return self.on_event(API.Types.UPDATE_NEW_MESSAGE, filters=Filters.bot_command(command))

    async def send(self, query: BaseObject):
        if not self.__running:
            raise RuntimeError('Client not started')

        if self.debug:
            self.logger.debug(f">>>>> {query.ID} {query.json(by_alias=True)}")

        await self.loop.run_in_executor(None, self.__tdjson.send, query.dict(by_alias=True))

    async def request(
            self,
            query: BaseObject,
            *,
            request_id: str = None,
            request_timeout: int = None
    ) -> Optional[RequestResult]:
        if not bool(request_id):
            request_id = query.EXTRA.get('request_id') or uuid.uuid4().hex

        if request_timeout is None:
            request_timeout = 10

        query.EXTRA['request_id'] = request_id
        pending_request = PendingRequest(self, query)
        self.__pending_requests[request_id] = pending_request

        try:
            await self.send(query)
            await pending_request.wait(raise_exc=True, timeout=request_timeout)
        finally:
            if self.debug and bool(pending_request.update):
                self.logger.debug(f"<<<<< {pending_request.update.ID} {pending_request.update.json(by_alias=True)}")

        return pending_request.update

    async def execute(self, query: BaseObject) -> ExecuteResult:
        if not self.__running:
            raise RuntimeError('Client not started')

        result = await self.loop.run_in_executor(None, self.__tdjson.execute, query.dict(by_alias=True))
        result_object = BaseObject.read(result)

        if isinstance(result_object, Error):
            raise AioTDLibError(
                code=result_object.code,
                message=result_object.message
            )

        return result_object

    async def receive(self, timeout: float = 1.0) -> Optional[BaseObject]:
        if not self.__running:
            raise RuntimeError('Client not started')

        data = await self.loop.run_in_executor(None, self.__tdjson.receive, timeout)

        if not bool(data):
            return None

        try:
            return BaseObject.read(data)
        except Exception as e:
            self.logger.error(f'Unable to parse incoming update! {e}', exc_info=True)
            return None

    async def start(self) -> 'Client':
        self.logger.info('Starting client')
        self.logger.info(f'Session workdir: {self.files_directory}')

        # Preparing middlewares handlers
        self.__middlewares_handlers = list(reversed(self.__middlewares))

        # Setting up asyncio stuff
        self.loop = asyncio.get_running_loop()
        self.__stop_idle_event = asyncio.Event()

        # Starting updates loop
        self.__running = True
        self.loop.create_task(self.__updates_loop())

        # Initialize authorization process
        await self.authorize()
        return self

    async def idle(self, stop_signals: tuple = (signal.SIGINT, signal.SIGTERM, signal.SIGABRT, signal.SIGQUIT)):
        if not self.__running:
            raise ValueError('Client is already stopped')

        self.logger.info('Started Idling...')

        # Registering signals handlers for graceful shutdown
        for sig in stop_signals:
            self.loop.add_signal_handler(sig, lambda _s=sig: asyncio.create_task(self.__stop_signal_handler(sig)))

        try:
            await self.__stop_idle_event.wait()
        except asyncio.CancelledError:
            pass

        self.logger.info('Stop Idling...')

    async def stop(self):
        self.logger.info('Stopping telegram client...')
        await self.__close()

        self.__running = False
        self.__stop_idle_event.set()

    async def authorize(self):
        if bool(self.phone_number):
            self.logger.info('Authorization process has been started with phone')
        else:
            self.logger.info('Authorization process has been started with bot token')

        auth_actions = {
            None: self.__auth_start,
            API.Types.AUTHORIZATION_STATE_WAIT_TDLIB_PARAMETERS: self.__set_tdlib_parameters,
            API.Types.AUTHORIZATION_STATE_WAIT_ENCRYPTION_KEY: self.__check_database_encryption_key,
            API.Types.AUTHORIZATION_STATE_WAIT_PHONE_NUMBER: self.__set_authentication_phone_number_or_check_bot_token,
            API.Types.AUTHORIZATION_STATE_WAIT_CODE: self.__check_authentication_code,
            API.Types.AUTHORIZATION_STATE_WAIT_REGISTRATION: self.__register_user,
            API.Types.AUTHORIZATION_STATE_WAIT_PASSWORD: self.__check_authentication_password,
            API.Types.AUTHORIZATION_STATE_READY: self.__auth_completed,
            API.Types.AUTHORIZATION_STATE_LOGGING_OUT: self.__auth_logging_out,
            API.Types.AUTHORIZATION_STATE_CLOSING: self.__auth_closing,
            API.Types.AUTHORIZATION_STATE_CLOSED: self.__auth_closed,
        }

        while not self.__is_authorized:
            result = None

            while True:
                try:
                    self.logger.debug('Current authorization state: %s', self.__current_authorization_state)
                    next_action = auth_actions.get(self.__current_authorization_state)

                    if bool(next_action):
                        result = await next_action()
                    else:
                        self.logger.error(f'Unhandled authorization state: {self.__current_authorization_state}')
                        break
                except AioTDLibError as e:
                    # Need to retry previous step
                    self.logger.error(e)
                    await asyncio.sleep(0.1)
                else:
                    break

            if isinstance(result, UpdateAuthorizationState):
                self.__current_authorization_state = result.authorization_state.ID

            await asyncio.sleep(0.1)

    # Cache related methods
    async def get_option_value(self, name: str) -> typing.Union[str, int, bool, None]:
        """
        Returns the value of an option by its name.
        (Check the list of available options on https://core.telegram.org/tdlib/options.)
        Can be called before authorization

        Params:
            name (:class:`str`)
                The name of the option
        """
        return await self.cache.get_option_value(name)

    async def get_main_list_chats(self, limit: int = 25) -> list[Chat]:
        """
        Returns an ordered list of chats in a main chat list.
        Chats are sorted by the pair (chat.position.order, chat.id) in descending order
        """
        return await self.cache.get_main_list_chats(limit)

    async def get_chat(self, chat_id: int) -> Chat:
        """
        Returns information about a chat by its identifier, this is an offline request if the current user is not a bot
        """
        return await self.cache.get_chat(chat_id)

    async def get_user(self, user_id: int) -> User:
        return await self.cache.get_user(user_id)

    async def get_user_full_info(self, user_id: int) -> UserFullInfo:
        return await self.cache.get_user_full_info(user_id)

    async def get_basic_group(self, basic_group_id: int) -> BasicGroup:
        return await self.cache.get_basic_group(basic_group_id)

    async def get_basic_group_full_info(self, basic_group_id: int) -> BasicGroupFullInfo:
        return await self.cache.get_basic_group_full_info(basic_group_id)

    async def get_supergroup(self, supergroup_id: int) -> BasicGroup:
        return await self.cache.get_basic_group(supergroup_id)

    async def get_supergroup_full_info(self, supergroup_id: int) -> BasicGroupFullInfo:
        return await self.cache.get_basic_group_full_info(supergroup_id)

    async def get_secret_chat(self, secret_chat_id: int) -> SecretChat:
        return await self.cache.get_secret_chat(secret_chat_id)

    async def get_chat_info(self, chat: Union[int, Chat], *, full: bool = False) -> Optional[ChatInfo]:
        chat = await self.get_chat(chat) if isinstance(chat, int) else chat

        if isinstance(chat.type_, ChatTypePrivate):
            if full:
                return await self.get_user_full_info(chat.type_.user_id)
            else:
                return await self.get_user(chat.type_.user_id)

        if isinstance(chat.type_, ChatTypeBasicGroup):
            if full:
                return await self.get_basic_group_full_info(chat.type_.basic_group_id)
            else:
                return await self.get_basic_group(chat.type_.basic_group_id)

        if isinstance(chat.type_, ChatTypeSupergroup):
            if full:
                return await self.get_supergroup_full_info(chat.type_.supergroup_id)
            else:
                return await self.get_supergroup(chat.type_.supergroup_id)

        if isinstance(chat.type_, ChatTypeSecret):
            return await self.get_secret_chat(chat.type_.secret_chat_id)

        raise ValueError(f'Unknown chat.type {chat.type_.__class__.__qualname__}')

    # API methods shorthands
    async def parse_text(self, text: str, *, parse_mode: str = None) -> Optional[FormattedText]:
        """
        Parses Bold, Italic, Underline, Strikethrough, Code, Pre, PreCode,
        TextUrl and MentionName entities contained in the text.
        """
        if text is None:
            return None

        # ParseTextEntities can be called synchronously
        if parse_mode is None:
            parse_mode = self.parse_mode
        else:
            parse_mode = parse_mode.lower()

        return await self.execute(
            ParseTextEntities(
                text=text,
                parse_mode=(
                    TextParseModeHTML()
                    if parse_mode == 'html' else
                    TextParseModeMarkdown(version=2)
                )
            )
        )

    async def __send_message(
            self,
            chat_id: int,
            content: InputMessageContent,
            *,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a text message. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            content (:class:`InputMessageContent`)
                The content of the message to be sent

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if bool(send_date):
            scheduling_state = MessageSchedulingStateSendAtDate(send_date=send_date)
        elif send_when_online:
            scheduling_state = MessageSchedulingStateSendWhenOnline()
        else:
            scheduling_state = None

        return await self.api.send_message(
            chat_id=chat_id,
            message_thread_id=0,
            reply_to_message_id=reply_to_message_id,
            options=MessageSendOptions.construct(
                disable_notification=disable_notification,
                from_background=False,
                scheduling_state=scheduling_state
            ),
            reply_markup=reply_markup,
            input_message_content=content,
            skip_validation=True,
            request_timeout=request_timeout
        )

    async def send_text(
            self,
            chat_id: int,
            text: str,
            *,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            disable_web_page_preview: bool = False,
            clear_draft: bool = True,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ) -> Message:
        """
        Sends a text message. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            text (:class:`str`)
                The text

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            disable_web_page_preview (:class:`bool`)
                True, if rich web page previews for URLs in the message text should be disabled

            clear_draft (:class:`bool`)
                True, if a chat message draft should be deleted

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """
        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageText.construct(
                text=(await self.parse_text(text)),
                disable_web_page_preview=disable_web_page_preview,
                clear_draft=clear_draft,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def edit_text(
            self,
            chat_id: int,
            message_id: int,
            text: str,
            *,
            reply_markup: ReplyMarkup = None,
            disable_web_page_preview: bool = False,
            clear_draft: bool = True,
            request_timeout: int = None
    ):
        """
        Edits the text of a message (or a text of a game message).
        Returns the edited message after the edit is completed on the server side

        Params:
            chat_id (:class:`int`)
                The chat the message belongs to

            message_id (:class:`int`)
                Identifier of the message

            text (:class:`str`)
                New text of the message

            reply_markup (:class:`ReplyMarkup`)
                The new message reply markup; for bots only

            disable_web_page_preview (:class:`bool`)
                True, if rich web page previews for URLs in the message text should be disabled

            clear_draft (:class:`bool`)
                True, if a chat message draft should be deleted

        """
        formatted_text = await self.parse_text(text)

        return await self.api.edit_message_text(
            chat_id=chat_id,
            message_id=message_id,
            reply_markup=reply_markup,
            input_message_content=InputMessageText.construct(
                text=formatted_text,
                disable_web_page_preview=disable_web_page_preview,
                clear_draft=clear_draft,
            ),
            skip_validation=True,
            request_timeout=request_timeout
        )

    async def send_photo(
            self,
            chat_id: int,
            photo: str,
            *,
            caption: str = None,
            added_sticker_file_ids: list[int] = None,
            photo_width: int = None,
            photo_height: int = None,
            ttl: int = None,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a photo with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            photo (:class:`str`)
                Photo to send. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Photo caption

            added_sticker_file_ids (:obj:`list[int]`)
                File identifiers of the stickers added to the photo, if applicable

            photo_width (:class:`int`)
                Photo width

            photo_height (:class:`int`)
                Photo height

            ttl (:class:`int`)
                Photo TTL (Time To Live), in seconds (0-60). A non-zero TTL can be specified only in private chats

            thumbnail (:class:`str`)
                Thumbnail file to send. Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(photo):
            photo_input_file = InputFileLocal(path=photo)
        else:
            photo_input_file = InputFileRemote(id=photo)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessagePhoto.construct(
                caption=(await self.parse_text(caption)),
                photo=photo_input_file,
                thumbnail=thumbnail,
                added_sticker_file_ids=added_sticker_file_ids,
                width=photo_width,
                height=photo_height,
                ttl=ttl,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_video(
            self,
            chat_id: int,
            video: str,
            *,
            caption: str = None,
            added_sticker_file_ids: list[int] = None,
            duration: int = None,
            video_width: int = None,
            video_height: int = None,
            ttl: int = None,
            supports_streaming: bool = False,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a video with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            video (:class:`str`)
                Video to send. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Video caption

            added_sticker_file_ids (:obj:`list[int]`)
                File identifiers of the stickers added to the photo, if applicable

            duration (:class:`int`)
                Duration of the video, in seconds

            video_width (:class:`int`)
                Video width

            video_height (:class:`int`)
                Video height

            ttl (:class:`int`)
                Video TTL (Time To Live), in seconds (0-60). A non-zero TTL can be specified only in private chats

            supports_streaming (:class:`bool`)
                True, if the video should be tried to be streamed

            thumbnail (:class:`str`)
                Thumbnail file to send. Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(video):
            video_input_file = InputFileLocal(path=video)
        else:
            video_input_file = InputFileRemote(id=video)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageVideo.construct(
                caption=(await self.parse_text(caption)),
                video=video_input_file,
                thumbnail=thumbnail,
                added_sticker_file_ids=added_sticker_file_ids,
                duration=duration,
                width=video_width,
                height=video_height,
                ttl=ttl,
                supports_streaming=supports_streaming,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_animation(
            self,
            chat_id: int,
            animation: str,
            *,
            caption: str = None,
            added_sticker_file_ids: list[int] = None,
            duration: int = None,
            animation_width: int = None,
            animation_height: int = None,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends an animation with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            animation (:class:`str`)
                Animation to send. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Animation caption

            added_sticker_file_ids (:obj:`list[int]`)
                File identifiers of the stickers added to the photo, if applicable

            duration (:class:`int`)
                Duration of the animation, in seconds

            animation_width (:class:`int`)
                Animation width

            animation_height (:class:`int`)
                Animation height

            thumbnail (:class:`str`)
                Thumbnail file to send. Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(animation):
            animation_input_file = InputFileLocal(path=animation)
        else:
            animation_input_file = InputFileRemote(id=animation)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageAnimation.construct(
                caption=(await self.parse_text(caption)),
                animation=animation_input_file,
                thumbnail=thumbnail,
                added_sticker_file_ids=added_sticker_file_ids,
                duration=duration,
                width=animation_width,
                height=animation_height,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_document(
            self,
            chat_id: int,
            document: str,
            *,
            caption: str = None,
            disable_content_type_detection: bool = False,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a document with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            document (:class:`str`)
                Document to be sent. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Document caption

            disable_content_type_detection (:class:`bool`)
                If true, automatic file type detection will be disabled and the document will be always sent as file.
                Always true for files sent to secret chats

            thumbnail (:class:`str`)
                Thumbnail file to send. Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(document):
            document_input_file = InputFileLocal(path=document)
        else:
            document_input_file = InputFileRemote(id=document)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageDocument.construct(
                document=document_input_file,
                caption=(await self.parse_text(caption)),
                thumbnail=thumbnail,
                disable_content_type_detection=disable_content_type_detection,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_audio(
            self,
            chat_id: int,
            audio: str,
            *,
            caption: str = None,
            duration: int = None,
            title: str = None,
            performer: str = None,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends an audio with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            audio (:class:`str`)
                Audio to be sent. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Audio caption

            duration (:class:`int`)
                Duration of the audio, in seconds; may be replaced by the server

            title (:class:`str`)
                Title of the audio; 0-64 characters; may be replaced by the server

            performer (:class:`str`)
                Performer of the audio; 0-64 characters, may be replaced by the server

            thumbnail (:class:`str`)
                Thumbnail of the cover for the album, if available.
                Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(audio):
            audio_input_file = InputFileLocal(path=audio)
        else:
            audio_input_file = InputFileRemote(id=audio)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageAudio.construct(
                audio=audio_input_file,
                caption=(await self.parse_text(caption)),
                album_cover_thumbnail=thumbnail,
                title=title,
                duration=duration,
                performer=performer
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_voice_note(
            self,
            chat_id: int,
            voice_note: str,
            *,
            caption: str = None,
            duration: int = None,
            waveform: str = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a voice note with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            voice_note (:class:`str`)
                Voice note to be sent. This parameter could be ether path to local file or remote file_id

            caption (:class:`str`)
                Voice note caption

            duration (:class:`int`)
                Duration of the audio, in seconds; may be replaced by the server

            waveform (:class:`str`)
                Waveform representation of the voice note, in 5-bit format

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(voice_note):
            voice_note_input_file = InputFileLocal(path=voice_note)
        else:
            voice_note_input_file = InputFileRemote(id=voice_note)

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageVoiceNote.construct(
                voice_note=voice_note_input_file,
                caption=(await self.parse_text(caption)),
                duration=duration,
                waveform=waveform
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def send_video_note(
            self,
            chat_id: int,
            video_note: str,
            *,
            duration: int = None,
            length: int = None,
            thumbnail: Union[str, InputThumbnail] = None,
            thumbnail_width: int = None,
            thumbnail_height: int = None,
            reply_to_message_id: int = None,
            reply_markup: ReplyMarkup = None,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None
    ):
        """
        Sends a video note with caption to chat. Returns the sent message

        Params:
            chat_id (:class:`int`)
                Target chat

            video_note (:class:`str`)
                Video note to be sent. This parameter could be ether path to local file or remote file_id

            duration (:class:`int`)
                Duration of the video, in seconds

            length (:class:`int`)
                Video width and height; must be positive and not greater than 640

            thumbnail (:class:`str`)
                Video note thumbnail, if available
                Sending thumbnails by file_id is currently not supported

            thumbnail_width (:class:`int`)
                Thumbnail width, usually shouldn't exceed 320. Use 0 if unknown

            thumbnail_height (:class:`int`)
                Thumbnail height, usually shouldn't exceed 320. Use 0 if unknown

            reply_to_message_id (:class:`int`)
                Identifier of the message to reply to or 0

            reply_markup (:class:`ReplyMarkup`)
                Markup for replying to the message; for bots only

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored

        """

        if os.path.exists(video_note):
            video_note_input_file = InputFileLocal(path=video_note)
        else:
            video_note_input_file = InputFileRemote(id=video_note)

        if isinstance(thumbnail, str):
            thumbnail = InputThumbnail.construct(
                # Sending thumbnails by file_id is currently not supported
                thumbnail=InputFileLocal(path=thumbnail),
                width=thumbnail_width,
                height=thumbnail_height,
            )

        return await self.__send_message(
            chat_id=chat_id,
            content=InputMessageVideoNote.construct(
                video_note=video_note_input_file,
                thumbnail=thumbnail,
                length=length,
                duration=duration,
            ),
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            disable_notification=disable_notification,
            send_when_online=send_when_online,
            send_date=send_date,
            request_timeout=request_timeout
        )

    async def forward_messages(
            self,
            from_chat_id: int,
            chat_id: int,
            message_ids: list[int],
            *,
            send_copy: bool = False,
            remove_caption: bool = False,
            disable_notification: bool = False,
            send_when_online: bool = False,
            send_date: int = None,
            request_timeout: int = None,
    ) -> Messages:
        """
        Forwards previously sent messages.
        Returns the forwarded messages in the same order as the message identifiers passed in message_ids.
        If a message can't be forwarded, null will be returned instead of the message

        Params:
            from_chat_id (:class:`int`)
                Identifier of the chat from which to forward messages

            chat_id (:class:`int`)
                Identifier of the chat to which to forward messages

            message_ids (:obj:`list[int]`)
                Identifiers of the messages to forward. Message identifiers must be in a strictly increasing order.
                At most 100 messages can be forwarded simultaneously

            send_copy (:class:`bool`)
                True, if content of the messages needs to be copied without links to the original messages.
                Always true if the messages are forwarded to a secret chat

            remove_caption (:class:`bool`)
                True, if media caption of message copies needs to be removed. Ignored if send_copy is false

            disable_notification (:class:`bool`)
                Pass true to disable notification for the message

            send_when_online: (:class:`bool`)
                When True, the message will be sent when the peer will be online.
                Applicable to private chats only and when the exact online status of the peer is known

            send_date: (:class:`int`)
                Date the message will be sent. The date must be within 367 days in the future.
                If send_date passed send_when_online will be ignored
        """
        if bool(send_date):
            scheduling_state = MessageSchedulingStateSendAtDate(send_date=send_date)
        elif send_when_online:
            scheduling_state = MessageSchedulingStateSendWhenOnline()
        else:
            scheduling_state = None

        return await self.api.forward_messages(
            chat_id=chat_id,
            from_chat_id=from_chat_id,
            message_ids=message_ids,
            options=MessageSendOptions.construct(
                disable_notification=disable_notification,
                from_background=False,
                scheduling_state=scheduling_state
            ),
            send_copy=send_copy,
            remove_caption=remove_caption,
            request_timeout=request_timeout,
            skip_validation=True
        )

    async def iter_chat_history(
            self,
            chat_id: int,
            from_message_id: int = 0,
            limit: int = None,
            only_local: bool = False
    ) -> AsyncIterator[Message]:
        """
        Iterates over messages in a chat.
        The messages are returned in a reverse chronological order (i.e., in order of decreasing message_id).
        Number of messages are limited by limit parameter

        Params:
            chat_id (:class:`int`)
                Chat identifier

            from_message_id (:class:`int`)
                Identifier of the message starting from which history must be fetched;
                use 0 to get results from the last message

            limit (:class:`int`)
                The maximum number of messages to be returned; must be positive
                If chat contains less than limit messages all of them would be returned

            only_local (:class:`bool`)
                If true, returns only messages that are available locally without sending network requests

        """

        if limit is None:
            limit = 100

        yielded_messages_count = 0
        need_finish = False
        request_limit = min(100, limit)

        while not need_finish:
            history = await self.api.get_chat_history(
                chat_id=chat_id,
                from_message_id=from_message_id,
                limit=request_limit,
                offset=0,
                only_local=only_local
            )

            if len(history.messages) == 0:
                break

            from_message_id = history.messages[-1].id

            for message in history.messages:
                yield message
                yielded_messages_count += 1

                if yielded_messages_count >= limit:
                    need_finish = True
                    break

            request_limit = min(100, limit - yielded_messages_count)

    async def _run(self):
        async with self:
            await self.idle()

    def run(self):
        asyncio.run(self._run())



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import MessageSender


class SetGroupCallParticipantVolumeLevel(BaseObject):
    """
    Changes volume level of a participant of an active group call. If the current user can manage the group call, then the participant's volume level will be changed for all users with default volume level
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        participant_id (:class:`MessageSender`)
            Participant identifier
        
        volume_level (:class:`int`)
            New participant's volume level; 1-20000 in hundreds of percents
        
    """

    ID: str = Field("setGroupCallParticipantVolumeLevel", alias="@type")
    group_call_id: int
    participant_id: MessageSender
    volume_level: int

    @staticmethod
    def read(q: dict) -> SetGroupCallParticipantVolumeLevel:
        return SetGroupCallParticipantVolumeLevel.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class CountryInfo(BaseObject):
    """
    Contains information about a country
    
    Params:
        country_code (:class:`str`)
            A two-letter ISO 3166-1 alpha-2 country code
        
        name (:class:`str`)
            Native name of the country
        
        english_name (:class:`str`)
            English name of the country
        
        is_hidden (:class:`bool`)
            True, if the country should be hidden from the list of all countries
        
        calling_codes (:obj:`list[str]`)
            List of country calling codes
        
    """

    ID: str = Field("countryInfo", alias="@type")
    country_code: str
    name: str
    english_name: str
    is_hidden: bool
    calling_codes: list[str]

    @staticmethod
    def read(q: dict) -> CountryInfo:
        return CountryInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetSuggestedFileName(BaseObject):
    """
    Returns suggested name for saving a file in a given directory
    
    Params:
        file_id (:class:`int`)
            Identifier of the file
        
        directory (:class:`str`)
            Directory in which the file is supposed to be saved
        
    """

    ID: str = Field("getSuggestedFileName", alias="@type")
    file_id: int
    directory: str

    @staticmethod
    def read(q: dict) -> GetSuggestedFileName:
        return GetSuggestedFileName.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ReorderInstalledStickerSets(BaseObject):
    """
    Changes the order of installed sticker sets
    
    Params:
        is_masks (:class:`bool`)
            Pass true to change the order of mask sticker sets; pass false to change the order of ordinary sticker sets
        
        sticker_set_ids (:obj:`list[int]`)
            Identifiers of installed sticker sets in the new correct order
        
    """

    ID: str = Field("reorderInstalledStickerSets", alias="@type")
    is_masks: bool
    sticker_set_ids: list[int]

    @staticmethod
    def read(q: dict) -> ReorderInstalledStickerSets:
        return ReorderInstalledStickerSets.construct(**q)
